----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    09:16:59 11/24/2012 
-- Design Name: 
-- Module Name:    EMAC_GXPCS - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.std_logic_misc.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity EMAC_GXPCS is
    Port (
					 reset : IN std_logic;
					 CLIENTCLK : IN std_logic; -- 2xSFP_REFCLK 312.5MHz norminal
					 RXUSRCLK : IN std_logic;
					 TXUSRCLK : IN std_logic;
					 RX_FSM_RESET_DONE : IN std_logic;
					 TX_FSM_RESET_DONE : IN std_logic;
           GTX_TXD : out  STD_LOGIC_VECTOR (31 downto 0);
           GTX_TXHEADER : out  STD_LOGIC_VECTOR (1 downto 0);
					 GTX_TX_READ : IN std_logic;
           GTX_RXD : in  STD_LOGIC_VECTOR (31 downto 0);
           GTX_RXDVLD : in  STD_LOGIC;
           GTX_RXHEADER : in  STD_LOGIC_VECTOR (1 downto 0);
           GTX_RXHEADERVLD : in  STD_LOGIC;
           GTX_RXGEARBOXSLIP_OUT : out  STD_LOGIC;
           GTX_RXGOOD : out  STD_LOGIC;
					 CLIENTEMACTXD : IN std_logic_vector(31 downto 0);
					 CLIENTEMACTXDBYTECNT : IN std_logic_vector(1 downto 0);
					 CLIENTEMACTXDVLD : IN std_logic;
					 CLIENTEMACTXUNDERRUN : IN std_logic;
					 EMACCLIENTRXD : OUT std_logic_vector(31 downto 0);
					 EMACCLIENTRXDVLD : OUT std_logic;
					 EMACCLIENTRXDBYTECNT : OUT std_logic_vector(1 downto 0);
					 EMACCLIENTRXGOODFRAME : OUT std_logic;
					 EMACCLIENTRXBADFRAME : OUT std_logic;
					 EMACCLIENTTXACK : OUT std_logic
					 );
end EMAC_GXPCS;

architecture Behavioral of EMAC_GXPCS is
COMPONENT BLOCK_SYNC_SM
	PORT(
		RXHEADER_IN : IN std_logic_vector(2 downto 0);
		RXHEADERVALID_IN : IN std_logic;
		USER_CLK : IN std_logic;
		SYSTEM_RESET : IN std_logic;          
		BLOCKSYNC_OUT : OUT std_logic;
		RXGEARBOXSLIP_OUT : OUT std_logic
		);
END COMPONENT;
COMPONENT EthernetCRCD32
	PORT(
		clk : IN std_logic;
		init : IN std_logic;
		ce : IN std_logic;
		d : IN std_logic_vector(31 downto 0);          
		byte_cnt : IN std_logic_vector(1 downto 0);          
		crc : OUT std_logic_vector(31 downto 0);
		bad_crc : OUT std_logic
		);
END COMPONENT;
COMPONENT SCRAMBLER
	PORT(
		UNSCRAMBLED_DATA_IN : IN std_logic_vector(31 downto 0);
		DATA_VALID_IN : IN std_logic;
		USER_CLK : IN std_logic;
		SYSTEM_RESET : IN std_logic;          
		SCRAMBLED_DATA_OUT : OUT std_logic_vector(31 downto 0)
		);
END COMPONENT;
COMPONENT RX_engine
	PORT(
		rxclk : IN std_logic;
		clientclk : IN std_logic;
		reset : IN std_logic;
		RX_FSM_RESET_DONE : IN std_logic;
		BLOCK_LOCK : IN std_logic;
		GTX_RXD : IN std_logic_vector(31 downto 0);
		GTX_RXDVLD : IN std_logic;
		GTX_RXHEADER : IN std_logic_vector(1 downto 0);
		GTX_RXHEADERVLD : IN std_logic;          
		DO : OUT std_logic_vector(31 downto 0);
		DOVLD : OUT std_logic;
		RxGoodFrame : OUT std_logic;
		RxBaddFrame : OUT std_logic;
		DOBYTECNT : OUT std_logic_vector(1 downto 0);
		GTX_RXGOOD : OUT std_logic;
		link_fault : OUT std_logic_vector(1 downto 0)
		);
END COMPONENT;
constant C_block : std_logic_vector(63 downto 0) := x"000000000000001e";
constant S0_block : std_logic_vector(63 downto 0) := x"d555555555555578";
constant C0S4_block : std_logic_vector(63 downto 0) := x"5555550000000033";
constant O0LS4_block : std_logic_vector(63 downto 0) := x"5555550001000066";
constant O0RS4_block : std_logic_vector(63 downto 0) := x"5555550002000066";
constant O0LC4_block : std_logic_vector(63 downto 0) := x"000000000100004b";
constant O0RC4_block : std_logic_vector(63 downto 0) := x"000000000200004b";
constant C0O4_block : std_logic_vector(63 downto 0) := x"000000000000002d";
constant T0_block : std_logic_vector(63 downto 0) := x"0000000000000087";
constant T1_block : std_logic_vector(63 downto 0) := x"0000000000000099";
constant T2_block : std_logic_vector(63 downto 0) := x"00000000000000aa";
constant T3_block : std_logic_vector(63 downto 0) := x"00000000000000b4";
constant T4_block : std_logic_vector(63 downto 0) := x"00000000000000cc";
constant T5_block : std_logic_vector(63 downto 0) := x"00000000000000d2";
constant T6_block : std_logic_vector(63 downto 0) := x"00000000000000e1";
constant T7_block : std_logic_vector(63 downto 0) := x"00000000000000ff";
constant EBLOCK : std_logic_vector(63 downto 0) := x"3c78f1e3c78f1e1e";
constant LBLOCK : std_logic_vector(63 downto 0) := x"0100000001000055";
constant RBLOCK : std_logic_vector(63 downto 0) := x"0200000002000055";
constant zero32 : std_logic_vector(31 downto 0) := (others => '0');
signal TX_strobe : std_logic := '0';
--signal TX_BUSY : std_logic := '0';
signal D_BLOCK : std_logic := '0';
signal PADDING : std_logic := '0';
signal TX_error : std_logic := '0';
signal TX_START : std_logic := '0';
signal TX_SFD : std_logic := '0';
signal TX_DATA : std_logic := '0';
signal TX_FCS : std_logic := '0';
signal TX_EFD : std_logic := '0';
signal TX_EFD_q : std_logic := '0';
signal TX_INIT : std_logic := '0';
signal TX_IDLE : std_logic := '0';
signal TX_header : std_logic_vector(1 downto 0) := (others => '0');
signal TX_BLOCK : std_logic_vector(63 downto 0) := (others => '0');
signal tx_byte_cnt_r : std_logic_vector(1 downto 0) := (others => '0');
signal reset_TXSyncRegs : std_logic_vector(2 downto 0) := (others => '0');
--type TX_FIFO_RAM is array(0 to 31) of std_logic_vector(33 downto 0);
--signal TX_FIFO : TX_FIFO_RAM;
signal TX_FIFO_DI : std_logic_vector(33 downto 0) := (others => '0');
signal TX_FIFO_DO : std_logic_vector(33 downto 0) := (others => '0');
signal TX_FIFO_wa : std_logic_vector(4 downto 0) := (others => '0');
signal TX_FIFO_ra : std_logic_vector(4 downto 0) := (others => '0');
--signal init_tx_crc : std_logic := '0';
signal PADDING_CNT : std_logic_vector(4 downto 0) := (others => '0');
signal tx_byte_cnt : std_logic_vector(1 downto 0) := (others => '0');
signal tx_word_cnt : std_logic_vector(8 downto 0) := (others => '0');
signal TXD : std_logic_vector(31 downto 0) := (others => '0');
signal TXD_r : std_logic_vector(31 downto 0) := (others => '0');
signal ce_tx_crc : std_logic := '0';
signal junboframe : std_logic := '0';
signal init_tx_crc : std_logic := '0';
signal tx_crc : std_logic_vector(31 downto 0) := (others => '0');
signal BLOCK_LOCK : std_logic := '0';
signal BLOCK_NOT_LOCK : std_logic := '0';
signal GTX_RXHEADER_IN : std_logic_vector(2 downto 0) := (others => '0');
signal TX_UNSCRAMBLED_DATA : std_logic_vector(31 downto 0) := (others => '0');
signal TX_UNSCRAMBLED_DATAVLD : std_logic := '0';
signal link_fault : std_logic_vector(1 downto 0) := (others => '0');
begin
EMACCLIENTTXACK <= TX_START;
process(CLIENTCLK)
begin
	if(CLIENTCLK'event and CLIENTCLK = '1')then
		TX_strobe <= not TX_strobe;
		if(reset = '1' or link_fault(1) = '1' or TX_FSM_RESET_DONE = '0')then
			TX_INIT <= '1';
		elsif(TX_strobe = '1')then
			TX_INIT <= '0';
		end if;
		if(TX_IDLE = '1')then
			PADDING_CNT <= "00001";
		elsif(PADDING_CNT(4) = '0')then
			PADDING_CNT <= PADDING_CNT + 1;
		end if;
		if(PADDING_CNT(4) = '1')then
			tx_byte_cnt <= CLIENTEMACTXDBYTECNT;
			TXD <= CLIENTEMACTXD;
		else
			tx_byte_cnt <= "00";
			if(CLIENTEMACTXDVLD = '1')then
				case CLIENTEMACTXDBYTECNT is
					when "01" => TXD <= zero32(31 downto 8) & CLIENTEMACTXD(7 downto 0);
					when "10" => TXD <= zero32(31 downto 16) & CLIENTEMACTXD(15 downto 0);
					when "11" => TXD <= zero32(31 downto 24) & CLIENTEMACTXD(23 downto 0);
					when others => TXD <= CLIENTEMACTXD;
				end case;
			else
				TXD <= (others => '0'); -- padding
			end if;
		end if;
		if(TX_START = '1')then
			ce_tx_crc <= '1';
		elsif(TX_IDLE = '1' or (CLIENTEMACTXDVLD = '0' and PADDING_CNT(4) = '1'))then
			ce_tx_crc <= '0';
		end if;
		TX_DATA <= ce_tx_crc;
		if(ce_tx_crc = '0')then
			tx_word_cnt <= (others => '0');
		else
			tx_word_cnt <= tx_word_cnt + 1;
		end if;
		if(CLIENTEMACTXUNDERRUN = '1' or (ce_tx_crc = '1' and tx_word_cnt = "101111010" and tx_byte_cnt(1) = tx_byte_cnt(0)))then
			TX_error <= '1';
		elsif(TX_strobe = '1')then
			TX_error <= '0';
		end if;
		if(TX_IDLE = '1' and CLIENTEMACTXDVLD = '1')then
			TX_START <= '1';
		else
			TX_START <= '0';
		end if;
		if(reset = '1' or TX_EFD = '1' or TX_EFD_q = '1')then
			TX_IDLE <= '1';
		elsif(CLIENTEMACTXDVLD = '1')then
			TX_IDLE <= '0';
		end if;
		if(TX_DATA = '0')then
			PADDING <= '0';
		elsif(CLIENTEMACTXDVLD = '0' and PADDING_CNT(4) = '0')then
			PADDING <= '1';
		end if;
		TX_SFD <= TX_START;
		TX_FCS <= TX_DATA and not ce_tx_crc;
		TX_EFD <= TX_FCS;
		TX_EFD_q <= TX_EFD;
		TXD_r <= TXD;
		if(TX_DATA = '1')then
			tx_byte_cnt_r <= tx_byte_cnt;
		end if;
		if(TX_strobe = '1')then
			D_BLOCK <= TX_DATA;
			if(link_fault(1) = '1')then
				TX_BLOCK <= RBLOCK;
			elsif(TX_INIT = '1')then
				TX_BLOCK <= LBLOCK;
			elsif(TX_error = '1')then
				TX_BLOCK <= EBLOCK;
			elsif(TX_START = '1')then
				TX_BLOCK <= C0S4_block;
			elsif(TX_SFD = '1')then
				TX_BLOCK <= S0_block;
			elsif(TX_DATA = '1')then
				case tx_byte_cnt_r is
					when "01" => TX_BLOCK <= tx_crc(23 downto 0) & TXD_r(7 downto 0) & TX_BLOCK(63 downto 32);
					when "10" => TX_BLOCK <= tx_crc(15 downto 0) & TXD_r(15 downto 0) & TX_BLOCK(63 downto 32);
					when "11" => TX_BLOCK <= tx_crc(7 downto 0) & TXD_r(23 downto 0) & TX_BLOCK(63 downto 32);
					when others => TX_BLOCK <= TXD_r & TX_BLOCK(63 downto 32);
				end case;
			elsif(TX_FCS = '1')then
				case tx_byte_cnt_r is
					when "01" => TX_BLOCK <= zero32(15 downto 0) & tx_crc(31 downto 24) & TX_BLOCK(63 downto 32) & T5_block(7 downto 0);
					when "10" => TX_BLOCK <= zero32(7 downto 0) & tx_crc(31 downto 16) & TX_BLOCK(63 downto 32) & T6_block(7 downto 0);
					when "11" => TX_BLOCK <= tx_crc(31 downto 8) & TX_BLOCK(63 downto 32) & T7_block(7 downto 0);
					when others => TX_BLOCK <= tx_crc & TX_BLOCK(63 downto 32);
				end case;
			elsif(TX_EFD = '1')then
				case tx_byte_cnt_r is
					when "00" => TX_BLOCK <= zero32(23 downto 0) & TX_BLOCK(63 downto 32) & T4_block(7 downto 0);
					when others => TX_BLOCK <= zero32 & TX_BLOCK(63 downto 32);
				end case;
			elsif(TX_IDLE = '1')then
				TX_BLOCK <= zero32 & TX_BLOCK(63 downto 32);
			end if;
		else
			if(TX_SFD = '1')then
				TX_BLOCK(63 downto 32) <= S0_block(63 downto 32);
			elsif(TX_DATA = '1')then
				case tx_byte_cnt_r is
					when "01" => TX_BLOCK(63 downto 32) <= tx_crc(23 downto 0) & TXD_r(7 downto 0);
					when "10" => TX_BLOCK(63 downto 32) <= tx_crc(15 downto 0) & TXD_r(15 downto 0);
					when "11" => TX_BLOCK(63 downto 32) <= tx_crc(7 downto 0) & TXD_r(23 downto 0);
					when others => TX_BLOCK(63 downto 32) <= TXD_r;
				end case;
			elsif(TX_FCS = '1')then
				case tx_byte_cnt_r is
					when "01" => TX_BLOCK(63 downto 32) <= zero32(15 downto 0) & tx_crc(31 downto 24) & T1_block(7 downto 0);
					when "10" => TX_BLOCK(63 downto 32) <= zero32(7 downto 0) & tx_crc(31 downto 16) & T2_block(7 downto 0);
					when "11" => TX_BLOCK(63 downto 32) <= tx_crc(31 downto 8) & T3_block(7 downto 0);
					when others => TX_BLOCK(63 downto 32) <= tx_crc;
				end case;
			elsif(TX_EFD = '1')then
				case tx_byte_cnt_r is
					when "00" => TX_BLOCK(63 downto 32) <= T0_block(31 downto 0);
					when others => TX_BLOCK(63 downto 32) <= zero32;
				end case;
			elsif(TX_IDLE = '1')then
				TX_BLOCK(63 downto 32) <= C_block(31 downto 0);
			end if;
			TX_BLOCK(31 downto 0) <= TX_BLOCK(63 downto 32);
		end if;
		if(D_BLOCK = '0')then
			TX_FIFO_DI(33 downto 32) <= "10";
		else
			TX_FIFO_DI(33 downto 32) <= "01";
		end if;		
--		TX_FIFO(conv_integer(TX_FIFO_WA)) <= TX_FIFO_DI;
		if(reset = '1')then
			TX_FIFO_WA <= (others => '0');
		else
			TX_FIFO_WA(2 downto 0) <= TX_FIFO_WA(2 downto 0) + 1;
		end if;
	end if;
end process;
g_TX_FIFO: for i in 0 to 33 generate
   i_TX_FIFO : RAM32X1D
   port map (
      DPO => TX_FIFO_DO(i),     -- Read-only 1-bit data output
      SPO => open,     -- R/W 1-bit data output
      A0 => TX_FIFO_WA(0),       -- R/W address[0] input bit
      A1 => TX_FIFO_WA(1),       -- R/W address[1] input bit
      A2 => TX_FIFO_WA(2),       -- R/W address[2] input bit
      A3 => TX_FIFO_WA(3),       -- R/W address[3] input bit
      A4 => TX_FIFO_WA(4),       -- R/W address[4] input bit
      D => TX_FIFO_DI(i),         -- Write 1-bit data input
      DPRA0 => TX_FIFO_RA(0), -- Read-only address[0] input bit
      DPRA1 => TX_FIFO_RA(1), -- Read-only address[1] input bit
      DPRA2 => TX_FIFO_RA(2), -- Read-only address[2] input bit
      DPRA3 => TX_FIFO_RA(3), -- Read-only address[3] input bit
      DPRA4 => TX_FIFO_RA(4), -- Read-only address[4] input bit
      WCLK => CLIENTCLK,   -- Write clock input
      WE => '1'        -- Write enable input
   );
end generate;
process(TXUSRCLK)
begin
	if(TXUSRCLK'event and TXUSRCLK = '1')then
		reset_TXSyncRegs <= reset_TXSyncRegs(1 downto 0) & reset;
		if(reset_TXSyncRegs(2) = '1')then
			TX_FIFO_RA <= (others => '0');
		elsif(GTX_TX_READ = '1')then
			TX_FIFO_RA(2 downto 0) <= TX_FIFO_RA(2 downto 0) + 1;
		end if;
		GTX_TXHEADER <= TX_FIFO_DO(33 downto 32);
		GTX_TXD <= TX_FIFO_DO(31 downto 0);
	end if;
end process;
i_tx_CRC: EthernetCRCD32 PORT MAP(
		clk => CLIENTCLK,
		init => TX_START,
		ce => ce_tx_crc,
		d => TXD,
		byte_cnt => tx_byte_cnt,
		crc => tx_crc,
		bad_crc => open
	);
i_BLOCK_SYNC_SM: BLOCK_SYNC_SM PORT MAP(
		BLOCKSYNC_OUT => BLOCK_LOCK,
		RXGEARBOXSLIP_OUT => GTX_RXGEARBOXSLIP_OUT,
		RXHEADER_IN => GTX_RXHEADER_IN,
		RXHEADERVALID_IN => GTX_RXHEADERVLD,
		USER_CLK => RXUSRCLK,
		SYSTEM_RESET => reset
	);
GTX_RXHEADER_IN <= '0' & GTX_RXHEADER;
BLOCK_NOT_LOCK <= not BLOCK_LOCK;
i_SCRAMBLER: SCRAMBLER PORT MAP(
		UNSCRAMBLED_DATA_IN => TX_BLOCK(31 downto 0),
		SCRAMBLED_DATA_OUT => TX_FIFO_DI(31 downto 0),
		DATA_VALID_IN => '1',
		USER_CLK => CLIENTCLK,
		SYSTEM_RESET => BLOCK_NOT_LOCK
	);
I_RX_engine: RX_engine PORT MAP(
		rxclk => RXUSRCLK,
		clientclk => CLIENTCLK,
		reset => reset,
		RX_FSM_RESET_DONE => RX_FSM_RESET_DONE,
		BLOCK_LOCK => BLOCK_LOCK,
		GTX_RXD => GTX_RXD,
		GTX_RXDVLD => GTX_RXDVLD,
		GTX_RXHEADER => GTX_RXHEADER,
		GTX_RXHEADERVLD => GTX_RXHEADERVLD,
		DO => EMACCLIENTRXD,
		DOVLD => EMACCLIENTRXDVLD,
		RxGoodFrame => EMACCLIENTRXGOODFRAME,
		RxBaddFrame => EMACCLIENTRXBADFRAME,
		DOBYTECNT => EMACCLIENTRXDBYTECNT,
		GTX_RXGOOD => GTX_RXGOOD,
		link_fault => link_fault
	);


end Behavioral;

