----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:58:44 12/06/2012 
-- Design Name: 
-- Module Name:    RX_engine - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.std_logic_misc.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity RX_engine is
    Port ( rxclk : in  STD_LOGIC;
           clientclk : in  STD_LOGIC;
           reset : in  STD_LOGIC; -- reset must last at least 6 clock cycles to clear the FIFO properly
           RX_FSM_RESET_DONE : in  STD_LOGIC;
           BLOCK_LOCK : in  STD_LOGIC;
           GTX_RXD : in  STD_LOGIC_VECTOR (31 downto 0);
           GTX_RXDVLD : in  STD_LOGIC;
           GTX_RXHEADER : in  STD_LOGIC_VECTOR (1 downto 0);
           GTX_RXHEADERVLD : in  STD_LOGIC;
           DO : out  STD_LOGIC_VECTOR (31 downto 0);
           DOVLD : out  STD_LOGIC;
           RxGoodFrame : out  STD_LOGIC;
           RxBaddFrame : out  STD_LOGIC;
           DOBYTECNT : out  STD_LOGIC_VECTOR (1 downto 0);
					 GTX_RXGOOD : out  STD_LOGIC;
           link_fault : out  STD_LOGIC_VECTOR (1 downto 0));
end RX_engine;

architecture Behavioral of RX_engine is
COMPONENT DESCRAMBLER
	PORT(
		SCRAMBLED_DATA_IN : IN std_logic_vector(31 downto 0);
		DATA_VALID_IN : IN std_logic;
		USER_CLK : IN std_logic;
		SYSTEM_RESET : IN std_logic;          
		UNSCRAMBLED_DATA_OUT : OUT std_logic_vector(31 downto 0)
		);
END COMPONENT;
COMPONENT EthernetCRCD32
	PORT(
		clk : IN std_logic;
		init : IN std_logic;
		ce : IN std_logic;
		d : IN std_logic_vector(31 downto 0);          
		byte_cnt : IN std_logic_vector(1 downto 0);          
		crc : OUT std_logic_vector(31 downto 0);
		bad_crc : OUT std_logic
		);
END COMPONENT;
COMPONENT link_status
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		send_LBLOCK : IN std_logic;
		rxdvld : IN std_logic;
		rxsync : IN std_logic_vector(1 downto 0);
		E_R : IN std_logic_vector(7 downto 0);
		local_fault : IN std_logic_vector(1 downto 0);
		remote_fault : IN std_logic_vector(1 downto 0);
		link_fault : OUT std_logic_vector(1 downto 0)
		);
END COMPONENT;
constant C_block : std_logic_vector(63 downto 0) := x"000000000000001e";
constant S0_block : std_logic_vector(63 downto 0) := x"d555555555555578";
constant C0S4_block : std_logic_vector(63 downto 0) := x"5555550000000033";
constant O0LS4_block : std_logic_vector(63 downto 0) := x"5555550001000066";
constant O0RS4_block : std_logic_vector(63 downto 0) := x"5555550002000066";
constant O0LC4_block : std_logic_vector(63 downto 0) := x"000000000100004b";
constant O0RC4_block : std_logic_vector(63 downto 0) := x"000000000200004b";
constant C0O4_block : std_logic_vector(63 downto 0) := x"000000000000002d";
constant T0_block : std_logic_vector(63 downto 0) := x"0000000000000087";
constant T1_block : std_logic_vector(63 downto 0) := x"0000000000000099";
constant T2_block : std_logic_vector(63 downto 0) := x"00000000000000aa";
constant T3_block : std_logic_vector(63 downto 0) := x"00000000000000b4";
constant T4_block : std_logic_vector(63 downto 0) := x"00000000000000cc";
constant T5_block : std_logic_vector(63 downto 0) := x"00000000000000d2";
constant T6_block : std_logic_vector(63 downto 0) := x"00000000000000e1";
constant T7_block : std_logic_vector(63 downto 0) := x"00000000000000ff";
constant E_block : std_logic_vector(63 downto 0) := x"3c78f1e3c78f1e1e";
constant L_block : std_logic_vector(63 downto 0) := x"0100000001000055";
constant R_block : std_logic_vector(63 downto 0) := x"0200000002000055";
constant zero32 : std_logic_vector(31 downto 0) := (others => '0');
type type_state is (RX_INIT, RX_C, RX_D, RX_T, RX_E);
signal state : type_state := RX_INIT;
type type_R is (C,S,D,T,E);
signal RX_TYPE : type_R := C;
type sr2 is array (3 downto 0) of std_logic_vector(1 downto 0);
signal byte_cnt_dl : sr2;
signal RXGOOD_cntr : std_logic_vector(4 downto 0) := (others => '0');
signal ce_rx_crc : std_logic := '0';
signal init_rx_crc : std_logic := '0';
signal E_R : std_logic_vector(7 downto 0) := (others => '0');
signal O4 : std_logic := '0';
signal D_R : std_logic := '0';
signal S0_R : std_logic := '0';
signal S0_R_r : std_logic := '0';
signal C0S4_R : std_logic := '0';
signal O0S4_R : std_logic := '0';
signal O0C4_R : std_logic := '0';
signal O0O4_R : std_logic := '0';
signal C0O4_R : std_logic := '0';
signal C_R : std_logic := '0';
signal T_R : std_logic_vector(7 downto 0) := (others => '0');
signal RXD : std_logic_vector(31 downto 0) := (others => '0');
signal send_LBLOCK : std_logic := '0';
signal RXDVLD : std_logic := '0';
signal RXDVLDp : std_logic := '0';
signal RXHEADERVLD : std_logic := '0';
signal S4_SFD : std_logic := '0';
signal en_ce_rx_crc : std_logic := '0';
signal en_ce_rx_crc_dl : std_logic_vector(3 downto 0) := (others => '0');
signal reset_RXSyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal RXSYNC : std_logic_vector(1 downto 0) := (others => '0');
signal RXHEADER : std_logic_vector(1 downto 0) := (others => '0');
signal rx_crc_d : std_logic_vector(31 downto 0) := (others => '0');
signal rx_crc_byte_cnt : std_logic_vector(1 downto 0) := (others => '0');
signal rx_data_byte_cnt : std_logic_vector(1 downto 0) := (others => '0');
signal bad_crc : std_logic := '0';
signal crc_OK : std_logic := '0';
signal check_crc : std_logic := '0';
signal check_crc_dl : std_logic_vector(2 downto 0) := (others => '0');
signal bad_frame : std_logic := '0';
signal RX_FIFO_DI : std_logic_vector(33 downto 0) := (others => '0');
signal RX_FIFO_DO : std_logic_vector(33 downto 0) := (others => '0');
signal RX_FIFO_wa : std_logic_vector(4 downto 0) := (others => '0');
signal RX_FIFO_ra : std_logic_vector(4 downto 0) := (others => '0');
--signal bad_block_r : std_logic := '0';
signal local_fault : std_logic_vector(1 downto 0) := (others => '0');
signal remote_fault : std_logic_vector(1 downto 0) := (others => '0');
signal col_cnt : std_logic_vector(7 downto 0) := (others => '0');
signal seq_cnt : std_logic_vector(1 downto 0) := (others => '0');
signal last_seq_type : std_logic_vector(1 downto 0) := (others => '0');
signal rxdata : std_logic_vector(31 downto 0) := (others => '0');
signal rxdata_wa : std_logic_vector(3 downto 0) := (others => '0');
signal rxdata_ra : std_logic_vector(3 downto 0) := (others => '0');
signal we_rxctrl : std_logic := '0';
signal re_rxctrl : std_logic := '0';
signal rxctrl_i : std_logic_vector(3 downto 0) := (others => '0');
signal rxctrl_o : std_logic_vector(3 downto 0) := (others => '0');
signal rxctrl_wa : std_logic_vector(3 downto 0) := (others => '0');
signal rxctrl_wc : std_logic_vector(2 downto 0) := (others => '0');
signal rxctrl_wa0SyncRegs : std_logic_vector(3 downto 0) := (others => '0');

begin
process(rxclk)
begin
	if(rxclk'event and rxclk = '1')then
		reset_RXSyncRegs <= reset_RXSyncRegs(1 downto 0) & reset;
		RXHEADERVLD <= GTX_RXHEADERVLD;
		if(reset_RXSyncRegs(2) = '1' or or_reduce(E_R) = '1')then
			GTX_RXGOOD <= '0';
		elsif(RXGOOD_cntr(4) = '1')then
			GTX_RXGOOD <= '1';
		end if;
		if(reset_RXSyncRegs(2) = '1' or or_reduce(E_R) = '1')then
			RXGOOD_cntr <= (others => '0');
		elsif(RXGOOD_cntr(4) = '0')then
			RXGOOD_cntr <= RXGOOD_cntr + 1;
		end if;
		if(RXHEADERVLD = '1')then
			if(or_reduce(E_R) = '1')then
				RX_TYPE <= E;
			elsif(C_R = '1' or C0O4_R = '1' or O0O4_R = '1' or O0C4_R = '1')then
				RX_TYPE <= C;
			elsif(S0_R = '1' or C0S4_R = '1' or O0S4_R = '1')then
				RX_TYPE <= S;
			elsif(D_R = '1')then
				RX_TYPE <= D;
			else
				RX_TYPE <= T;
			end if;
		end if;
		if(reset_RXSyncRegs(2) = '1' or RX_FSM_RESET_DONE = '0' or BLOCK_LOCK = '0')then
			state <= RX_INIT;
			send_LBLOCK <= '1';
		elsif(RXHEADERVLD = '1')then
			case state is
				when RX_INIT =>
					if(RX_TYPE = E or RX_TYPE = D or RX_TYPE = T)then
						state <= RX_E;
						bad_frame <= '1';
					elsif(RX_TYPE = S)then
						state <= RX_D;
						bad_frame <= '0';
					else
						state <= RX_C;
					end if;
					send_LBLOCK <= '0';
				when RX_C =>
					if(RX_TYPE = E or RX_TYPE = D or RX_TYPE = T)then
						state <= RX_E;
						bad_frame <= '1';
					elsif(RX_TYPE = S)then
						state <= RX_D;
						bad_frame <= '0';
					end if;
				when RX_D =>
					if(RX_TYPE = T)then
						if(or_reduce(E_R) = '0' and D_R = '0' and or_reduce(T_R) = '0')then
							state <= RX_T;
						else
							state <= RX_E;
							bad_frame <= '1';
						end if;
					elsif(RX_TYPE /= D)then
						state <= RX_E;
						bad_frame <= '1';
					end if;
				when RX_T =>
					if(RX_TYPE = C)then
						state <= RX_C;
					elsif(RX_TYPE = S)then
						state <= RX_D;
						bad_frame <= '0';
					end if;
				when RX_E =>
					if(RX_TYPE = C)then
						state <= RX_C;
					elsif(RX_TYPE = D)then
						state <= RX_D;
					elsif(RX_TYPE = T and or_reduce(E_R) = '0' and D_R = '0' and or_reduce(T_R) = '0')then
						state <= RX_T;
					end if;
				when others => null;
			end case;
		end if;
	end if;
end process;
process(rxclk)
begin
	if(rxclk'event and rxclk = '1')then
		if(GTX_RXHEADERVLD = '0')then
			RXSYNC <= "00";
		else
			RXSYNC <= GTX_RXHEADER;
		end if;
		if(GTX_RXHEADERVLD = '1')then
			RXHEADER <= GTX_RXHEADER;
		end if;
		RXDVLD <= GTX_RXDVLD;
		RXDVLDp <= GTX_RXDVLD;
		if(RXDVLD = '1')then
			if(RXHEADER(1) = '1' and (C0S4_R = '1' or O0S4_R = '1'))then
				S4_SFD <= '1';
			else
				S4_SFD <= '0';
			end if;
			if((RXHEADER(1) = '1' and S0_R = '1') or S4_SFD = '1')then
				init_rx_crc <= '1';
			else
				init_rx_crc <= '0';
			end if;
			if(RXSYNC(1) = '1')then
				rx_crc_d <= RXD(7 downto 0) & RXD(31 downto 8); 
			else
				rx_crc_d <= RXD; 
			end if;
			if(RXHEADER(0) = '1')then
				rx_crc_byte_cnt <= "00";
			elsif(RXSYNC(1) = '1')then
				case RXD(6 downto 4) is
					when "001" => rx_crc_byte_cnt <= "01";
					when "010" => rx_crc_byte_cnt <= "10";
					when others => rx_crc_byte_cnt <= "11";
				end case;
			else
				case rx_crc_d(29 downto 28) is
					when "00" => rx_crc_byte_cnt <= "01";
					when "01" => rx_crc_byte_cnt <= "10";
					when "10" => rx_crc_byte_cnt <= "11";
					when others => rx_crc_byte_cnt <= "00";
				end case;
			end if;
			en_ce_rx_crc_dl <= en_ce_rx_crc_dl(2 downto 0) & en_ce_rx_crc;
			check_crc_dl <= check_crc_dl(1 downto 0) & check_crc;
			if(reset_RXSyncRegs(2) = '1' or or_reduce(E_R) = '1' or (RXSYNC(1) = '1' and RXD(7 downto 4) = x"8") or or_reduce(T_R(4 downto 1)) = '1' or (or_reduce(T_R(7 downto 5)) = '1' and RXHEADERVLD = '1'))then
				en_ce_rx_crc <= '0';
			elsif(init_rx_crc = '1')then
				en_ce_rx_crc <= '1';
			end if;
			check_crc <= en_ce_rx_crc_dl(0) and not en_ce_rx_crc;
			if(check_crc = '1')then
				crc_OK <= not bad_crc;
			end if;
			if(RXHEADER(0) = '1')then
				rx_data_byte_cnt <= "00";
			elsif(RXSYNC(1) = '1')then
				case RXD(6 downto 4) is
					when "001" => rx_data_byte_cnt <= "01";
					when "010" => rx_data_byte_cnt <= "10";
					when "011" => rx_data_byte_cnt <= "11";
					when others => rx_data_byte_cnt <= "00";
				end case;
			else
				case rx_crc_d(30 downto 28) is
					when "111" => rx_data_byte_cnt <= "11";
					when "110" => rx_data_byte_cnt <= "10";
					when others => rx_data_byte_cnt <= "01";
				end case;
			end if;
			byte_cnt_dl <= byte_cnt_dl(2 downto 0) & rx_data_byte_cnt;
		end if;
		we_rxctrl <= en_ce_rx_crc_dl(3) and RXDVLD;
		if(reset_RXSyncRegs(2) = '1')then
			rxdata_wa <= (others => '0');
		elsif(ce_rx_crc = '1')then
			rxdata_wa <= rxdata_wa + 1;
		end if;
		if(reset_RXSyncRegs(2) = '1')then
			rxctrl_wa <= (others => '0');
		elsif(we_rxctrl = '1')then
			rxctrl_wa <= rxctrl_wa + 1;
		end if;
		if(RXHEADERVLD = '1')then
			if(RXHEADER(0) = RXHEADER(1))then -- valid block is either data or control
				E_R(0) <= '1';
			elsif(RXHEADER(1) = '1')then
-- if last block had valid S4, this data block must start with SFD
				if((C0S4_R = '1' or O0S4_R = '1') and RXD /= S0_block(63 downto 32))then
					E_R(0) <= '1';
				else
					E_R(0) <= '0';
				end if;
			elsif(D_R = '1' and RXD(7) = '0')then
				E_R(0) <= '1';
			else
				E_R(0) <= '0';
			end if;
		end if;
		if(RXDVLD = '1' and RXHEADERVLD = '0' and RXHEADER(0) = '1' )then
			E_R(7 downto 1) <= (others => '0');
			if(C_R = '0' and C0O4_R = '0' and C0S4_R = '0' and O0S4_R = '0' and O0O4_R = '0' and S0_R = '0' and O0C4_R = '0' and T_R = x"00")then
				E_R(1) <= '1';
			end if;
			if((C0O4_R = '1' or O0O4_R = '1') and RXD /= L_block(63 downto 32) and RXD /= R_block(63 downto 32))then
				E_R(2) <= '1';
			end if;
			if((C0S4_R = '1' or O0S4_R = '1') and (RXD(31 downto 8) /= C0S4_block(63 downto 40) or RXD(3 downto 0) /= C0S4_block(35 downto 32)))then
				E_R(3) <= '1';
			end if;
			if(S0_R = '1' and RXD /= S0_block(63 downto 32))then
				E_R(4) <= '1';
			end if;
			if((C_R = '1' or O0C4_R = '1' or T_R(2 downto 0) /= "000") and or_reduce(RXD) = '1')then
				E_R(5) <= '1';
			end if;
			if(T_R(3) = '1' and or_reduce(RXD(31 downto 4)) = '1')then
				E_R(6) <= '1';
			end if;
			if(T_R(4) = '1' and or_reduce(RXD(31 downto 11)) = '1')then
				E_R(6) <= '1';
			end if;
			if(T_R(5) = '1' and or_reduce(RXD(31 downto 18)) = '1')then
				E_R(7) <= '1';
			end if;
			if(T_R(6) = '1' and  or_reduce(RXD(31 downto 25)) = '1')then
				E_R(7) <= '1';
			end if;
		end if;
		if(RXHEADER = "01")then 
			if(RXHEADERVLD = '1')then
				D_R <= '1';
				LOCAL_FAULT <= "00";
				REMOTE_FAULT <= "00";
				C_R <= '0';
				C0O4_R <= '0';
				C0S4_R <= '0';
				O0S4_R <= '0';
				S0_R <= '0';
				O0C4_R <= '0';
				T_R <= x"00";
			end if;
		elsif(RXHEADER = "10")then
			if(RXSYNC(1) = '1')then
				D_R <= '0';
				if(RXD(3 downto 0) = x"5" or RXD(3 downto 0) = x"6" or RXD(3 downto 0) = x"b")then
					LOCAL_FAULT(1) <= RXD(24);
					REMOTE_FAULT(1) <= RXD(25);
				else
					LOCAL_FAULT(1) <= '0';
					REMOTE_FAULT(1) <= '0';
				end if;
				if(RXD(3 downto 0) = x"5" or RXD(3 downto 0) = x"d")then
					O4 <= '1';
				else
					O4 <= '0';
				end if;
				if(RXD = C_block(31 downto 0))then
					C_R <= '1';
				else
					C_R <= '0';
				end if;
				if(RXD = C0O4_block(31 downto 0))then
					C0O4_R <= '1';
				else
					C0O4_R <= '0';
				end if;
				if(RXD = C0S4_block(31 downto 0))then
					C0S4_R <= '1';
				else
					C0S4_R <= '0';
				end if;
				if(RXD = O0LS4_block(31 downto 0) or RXD = O0RS4_block(31 downto 0))then
					O0S4_R <= '1';
				else
					O0S4_R <= '0';
				end if;
				if(RXD = L_block(31 downto 0) or RXD = R_block(31 downto 0))then
					O0O4_R <= '1';
				else
					O0O4_R <= '0';
				end if;
				if(RXD = S0_block(31 downto 0))then
					S0_R <= '1';
				else
					S0_R <= '0';
				end if;
				if(RXD = O0LC4_block(31 downto 0) or RXD = O0RC4_block(31 downto 0))then
					O0C4_R <= '1';
				else
					O0C4_R <= '0';
				end if;
				if(RXD(7 downto 0) = T0_block(7 downto 0) and RXD(31 downto 15) = T0_block(31 downto 15))then
					T_R(0) <= '1';
				else
					T_R(0) <= '0';
				end if;
				if(RXD(7 downto 0) = T1_block(7 downto 0) and RXD(31 downto 22) = T1_block(31 downto 22))then
					T_R(1) <= '1';
				else
					T_R(1) <= '0';
				end if;
				if(RXD(7 downto 0) = T2_block(7 downto 0) and RXD(31 downto 29) = T2_block(31 downto 29))then
					T_R(2) <= '1';
				else
					T_R(2) <= '0';
				end if;
				if(RXD(7 downto 0) = T3_block(7 downto 0))then
					T_R(3) <= '1';
				else
					T_R(3) <= '0';
				end if;
				if(RXD(7 downto 0) = T4_block(7 downto 0))then
					T_R(4) <= '1';
				else
					T_R(4) <= '0';
				end if;
				if(RXD(7 downto 0) = T5_block(7 downto 0))then
					T_R(5) <= '1';
				else
					T_R(5) <= '0';
				end if;
				if(RXD(7 downto 0) = T6_block(7 downto 0))then
					T_R(6) <= '1';
				else
					T_R(6) <= '0';
				end if;
				if(RXD(7 downto 0) = T7_block(7 downto 0))then
					T_R(7) <= '1';
				else
					T_R(7) <= '0';
				end if;
			elsif(RXDVLD = '1')then
				if(O4 = '1')then
					LOCAL_FAULT(0) <= RXD(24);
					REMOTE_FAULT(0) <= RXD(25);
				else
					LOCAL_FAULT(0) <= '0';
					REMOTE_FAULT(0) <= '0';
				end if;
			end if;
		end if;
	end if;
end process;
i_link_status: link_status PORT MAP(
		clk => rxclk,
		reset => reset_RXSyncRegs(2),
		rxdvld => RXDVLD,
		rxsync => RXSYNC,
		E_R => E_R,
		local_fault => LOCAL_FAULT,
		remote_fault => REMOTE_FAULT,
		send_LBLOCK => send_LBLOCK,
		link_fault => link_fault
	);
i_rx_CRC: EthernetCRCD32 PORT MAP(
		clk => rxclk,
		init => init_rx_crc,
		ce => ce_rx_crc,
		d => rx_crc_d,
		byte_cnt => rx_crc_byte_cnt,
		crc => open,
		bad_crc => bad_crc
	);
ce_rx_crc <= RXDVLDp and en_ce_rx_crc;
i_DESCRAMBLER: DESCRAMBLER PORT MAP(
		SCRAMBLED_DATA_IN => GTX_RXD,
		UNSCRAMBLED_DATA_OUT => RXD,
		DATA_VALID_IN => GTX_RXDVLD,
		USER_CLK => rxclk,
		SYSTEM_RESET => not BLOCK_LOCK
	);
process(clientclk)
begin
	if(clientclk'event and clientclk = '1')then
		if(reset = '1')then
			rxdata_ra <= (others => '0');
--			rxctrl_ra <= (others => '0');
		elsif(re_rxctrl = '1')then
			rxdata_ra <= rxdata_ra + 1;
--			rxctrl_ra <= rxctrl_ra + 1;
		end if;
		if(reset = '1')then
			rxctrl_wa0SyncRegs <= (others => '0');
		else
			rxctrl_wa0SyncRegs <= rxctrl_wa0SyncRegs(2 downto 0) & rxctrl_wa(0);
		end if;
		if(reset = '1')then
			rxctrl_wc <= (others => '0');
		elsif(rxctrl_wa0SyncRegs(3) /= rxctrl_wa0SyncRegs(2) and re_rxctrl = '0')then
			rxctrl_wc <= rxctrl_wc + 1;
		elsif(rxctrl_wa0SyncRegs(3) = rxctrl_wa0SyncRegs(2) and re_rxctrl = '1')then
			rxctrl_wc <= rxctrl_wc - 1;
		end if;
		if(reset = '1' or (rxctrl_wc(2 downto 1) = "00" and (rxctrl_wc(0) = '0' or re_rxctrl = '1')) or rxctrl_o(3 downto 2) /= "00")then
			re_rxctrl <= '0';
		elsif(rxctrl_wc(2) = '1' OR rxctrl_wc(1 downto 0) = "11")then
			re_rxctrl <= '1';
		end if;
		if(reset = '1' or re_rxctrl = '0')then
			RxGoodFrame <= '0';
			RxBaddFrame <= '0';
		else
			RxGoodFrame <= rxctrl_o(3);
			RxBaddFrame <= rxctrl_o(2);
		end if;
		if(reset = '1' or re_rxctrl = '0' or rxctrl_o(3 downto 2) /= "00")then
			DOVLD <= '0';
			DOBYTECNT <= "00";
		else
			DOVLD <= '1';
			DOBYTECNT <= rxctrl_o(1 downto 0);
		end if;
		DO <= rxdata;
	end if;
end process;
g_rxdata: for i in 0 to 31 generate
   i_rxdata : RAM32X1D
   port map (
      DPO => rxdata(i),     -- Read-only 1-bit data output
      SPO => open,     -- R/W 1-bit data output
      A0 => rxdata_wa(0),       -- R/W address[0] input bit
      A1 => rxdata_wa(1),       -- R/W address[1] input bit
      A2 => rxdata_wa(2),       -- R/W address[2] input bit
      A3 => rxdata_wa(3),       -- R/W address[3] input bit
      A4 => '0',       -- R/W address[4] input bit
      D => rx_crc_d(i),         -- Write 1-bit data input
      DPRA0 => rxdata_ra(0), -- Read-only address[0] input bit
      DPRA1 => rxdata_ra(1), -- Read-only address[1] input bit
      DPRA2 => rxdata_ra(2), -- Read-only address[2] input bit
      DPRA3 => rxdata_ra(3), -- Read-only address[3] input bit
      DPRA4 => '0', -- Read-only address[4] input bit
      WCLK => rxclk,   -- Write clock input
      WE => ce_rx_crc        -- Write enable input
   );
end generate;
g_rxctrl: for i in 0 to 3 generate
   i_rxctrl : RAM32X1D
   port map (
      DPO => rxctrl_o(i),     -- Read-only 1-bit data output
      SPO => open,     -- R/W 1-bit data output
      A0 => rxctrl_wa(0),       -- R/W address[0] input bit
      A1 => rxctrl_wa(1),       -- R/W address[1] input bit
      A2 => rxctrl_wa(2),       -- R/W address[2] input bit
      A3 => rxctrl_wa(3),       -- R/W address[3] input bit
      A4 => '0',       -- R/W address[4] input bit
      D => rxctrl_i(i),         -- Write 1-bit data input
      DPRA0 => rxdata_ra(0), -- Read-only address[0] input bit
      DPRA1 => rxdata_ra(1), -- Read-only address[1] input bit
      DPRA2 => rxdata_ra(2), -- Read-only address[2] input bit
      DPRA3 => rxdata_ra(3), -- Read-only address[3] input bit
      DPRA4 => '0', -- Read-only address[4] input bit
      WCLK => rxclk,   -- Write clock input
      WE => we_rxctrl        -- Write enable input
   );
end generate;
rxctrl_i(3) <= check_crc_dl(2) and not bad_frame and crc_OK;
rxctrl_i(2) <= check_crc_dl(2) and (bad_frame or not crc_OK);
rxctrl_i(1 downto 0) <= byte_cnt_dl(3);
end Behavioral;

