----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    09:49:10 06/14/2012 
-- Design Name: 
-- Module Name:    TTS_if - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
-- TTC Hamming encoding
-- hmg[0] = d[0]^d[1]^d[2]^d[3];
-- hmg[1] = d[0]^d[4]^d[5]^d[6];
-- hmg[2] = d[1]^d[2]^d[4]^d[5]^d[7];
-- hmg[3] = d[1]^d[3]^d[4]^d[6]^d[7];
-- hmg[4] = d[0]^d[2]^d[3]^d[5]^d[6]^d[7];
--
-- L1A trigger from CTR module takes the position of L1A in TTC
-- message format takes that of TTC broadcast message.(FMT = 0)
-- BcntRes uses brcst_data(0), TTS signals uses brcst_data(7 downto 4), brcst_data(3 downto 1) are reserved and are always 0
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.std_logic_misc.all;
library UNISIM;
use UNISIM.VComponents.all;

entity TTS_if is
    Port ( sysclk : in  STD_LOGIC;
           CDRclk : in  STD_LOGIC;
					 local_TTC : in  STD_LOGIC;
           TTC_Ready : in  STD_LOGIC;
           TTC_strobe : in  STD_LOGIC; -- TTC_strobe is clocked by CDRclk, it is one CDR_clk cycle ahead of TTC_clk rising edge
           TTS : in  STD_LOGIC_VECTOR (3 downto 0); -- this is clocked by sysclk, the state signal in the top module AMC13_T1
           BC0 : in  STD_LOGIC; -- use delayed BC0
           L1A_trig : in  STD_LOGIC;
           TTS_out_p : out  STD_LOGIC;
           TTS_out_n : out  STD_LOGIC);
end TTS_if;

architecture Behavioral of TTS_if is
signal TTS_out : std_logic := '0';
signal TTS_out_q : std_logic := '0';
signal inh_msg : std_logic := '0';
signal TTS_fifo_empty : std_logic := '1';
signal SendBC0 : std_logic := '0';
signal start : std_logic := '0';
signal stop : std_logic := '0';
signal busy : std_logic := '0';
signal Bcnt : std_logic_vector(11 downto 0) := (others =>'0');
signal sr : std_logic_vector(14 downto 0) := (others =>'1');
signal msgHmg : std_logic_vector(4 downto 0) := (others =>'0');
signal phase : std_logic_vector(1 downto 0) := (others =>'0');
signal phase_l : std_logic_vector(1 downto 0) := (others =>'0');
signal TTS_sync_wa : std_logic_vector(1 downto 0) := (others =>'0');
signal TTS_sync_ra : std_logic_vector(1 downto 0) := (others =>'0');
signal TTS_sync_wa_sync : std_logic_vector(1 downto 0) := (others =>'0');
signal TTS_sync : std_logic_vector(3 downto 0) := (others =>'0');
signal TTS_old : std_logic_vector(3 downto 0) := (others =>'0');
signal TTS_fifo_a : std_logic_vector(4 downto 0) := (others =>'1');
signal TTS_fifo_Do : std_logic_vector(3 downto 0) := (others =>'0');
signal TTS_fifo_Do_q : std_logic_vector(3 downto 0) := (others =>'0');
signal TTS_fifo_we : std_logic := '0';
signal TTS_fifo_re : std_logic := '0';
component icon2
  PORT (
    CONTROL0 : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
    CONTROL1 : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0));

end component;
component ila64x4096
  PORT (
    CONTROL : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
    CLK : IN STD_LOGIC;
    DATA : IN STD_LOGIC_VECTOR(63 DOWNTO 0);
    TRIG0 : IN STD_LOGIC_VECTOR(7 DOWNTO 0));

end component;
component ila384_8
  PORT (
    CONTROL : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
    CLK : IN STD_LOGIC;
    DATA : IN STD_LOGIC_VECTOR(383 DOWNTO 0);
    TRIG0 : IN STD_LOGIC_VECTOR(7 DOWNTO 0));

end component;
component vio_async_in256
  PORT (
    CONTROL : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
    ASYNC_IN : IN STD_LOGIC_VECTOR(255 DOWNTO 0));

end component;
signal cs0_control : std_logic_vector(35 downto 0) := (others => '0');
signal cs1_control : std_logic_vector(35 downto 0) := (others => '0');
signal cs0_data : std_logic_vector(63 downto 0) := (others => '0');
signal ASYNC_IN : std_logic_vector(255 downto 0) := (others => '0');
signal cs0_trig : std_logic_vector(7 downto 0) := (others => '0');
begin
--icon : icon2
--  port map (
--    CONTROL0 => cs0_control,
--    CONTROL1 => cs1_control);
--cs0 : ila64x4096
--  port map (
--    CONTROL => cs0_control,
--    CLK => CDRclk,
--    DATA => cs0_data,
--    TRIG0 => cs0_trig);
--cs0_data(0) <= 
--vio : vio_async_in256
--  port map (
--    CONTROL => cs1_control,
--    ASYNC_IN => ASYNC_IN);
--ASYNC_IN(3 downto 0) <= TTS;
--ASYNC_IN(7 downto 4) <= TTS_sync;
-- even though the TTS_out LVDS output pins' polarity is reversed, it does not matter because of the coding scheme
i_TTS_out: OBUFDS generic map(IOSTANDARD => "LVDS_25") port map (O => TTS_out_p, OB => TTS_out_n, I => TTS_out_q);
g_tts_sync : for i in 0 to 3 generate
   i_ttc_sync : RAM32X1D
   port map (
      DPO => TTS_sync(i),     -- Read-only 1-bit data output
      A0 => TTS_sync_wa(0),       -- R/W address[0] input bit
      A1 => TTS_sync_wa(1),       -- R/W address[1] input bit
      A2 => '0',       -- R/W address[2] input bit
      A3 => '0',       -- R/W address[3] input bit
      A4 => '0',       -- R/W address[4] input bit
      D => TTS(i),         -- Write 1-bit data input
      DPRA0 => TTS_sync_ra(0), -- Read-only address[0] input bit
      DPRA1 => TTS_sync_ra(1), -- Read-only address[1] input bit
      DPRA2 => '0', -- Read-only address[2] input bit
      DPRA3 => '0', -- Read-only address[3] input bit
      DPRA4 => '0', -- Read-only address[4] input bit
      WCLK => sysclk,   -- Write clock input
      WE => '1'       -- Write enable input
   );
end generate;
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		TTS_sync_wa(1) <= TTS_sync_wa(0);
		TTS_sync_wa(0) <= not TTS_sync_wa(1);
	end if;
end process;
g_TTS_fifo: for i in 0 to 3 generate
   i_TTS_fifo : SRLC32E
   port map (
      Q => TTS_fifo_Do(i),        -- SRL data output
      Q31 => open,    -- SRL cascade output pin
      A => TTS_fifo_a,        -- 5-bit shift depth select input
      CE => TTS_fifo_we,      -- Clock enable input
      CLK => CDRclk,    -- Clock input
      D => TTS_sync(i)         -- SRL data input
   );
end generate;
process(CDRclk)
begin
	if(CDRclk'event and CDRclk = '1')then
		TTS_sync_wa_sync <= TTS_sync_wa;
		TTS_sync_ra(1) <= not TTS_sync_wa_sync(0);
		TTS_sync_ra(0) <= TTS_sync_wa_sync(1);
		if(TTC_Ready = '0')then
			TTS_old <= x"0";
			TTS_fifo_we <= '0';
		elsif(TTS_sync /= TTS_old and TTS_fifo_a /= "11110")then
			TTS_old <= TTS_sync;
			TTS_fifo_we <= '1';
		else
			TTS_fifo_we <= '0';
		end if;
		if(TTC_Ready = '0')then
			TTS_fifo_a <= (others => '1');
		elsif(TTS_fifo_we = '1' and TTS_fifo_re = '0')then
			TTS_fifo_a <= TTS_fifo_a + 1;
		elsif(TTS_fifo_we = '0' and TTS_fifo_re = '1' and TTS_fifo_empty = '0')then
			TTS_fifo_a <= TTS_fifo_a - 1;
		end if;
		TTS_fifo_re <= TTC_strobe and start;
		TTS_fifo_empty <= and_reduce(TTS_fifo_a);
		if(TTC_strobe = '1')then
			if(BC0 = '1')then
				Bcnt <= (others => '0');
			else
				Bcnt <= Bcnt + 1;
			end if;
			if(Bcnt = x"dc0")then -- decoder may need to delay the BcntRes signal to make L1A_trig align with the Bcnt
				inh_msg <= '1';
			elsif(SendBC0 = '1')then
				inh_msg <= '0';
			end if;
			if(and_reduce(TTS_fifo_a) = '0')then
				TTS_fifo_Do_q <= TTS_fifo_Do;
			end if;
			if(busy = '0' and ((TTS_fifo_empty = '0' and inh_msg = '0') or (SendBC0 = '1' and inh_msg = '1') or Bcnt(7 downto 0) = x"00"))then
				start <= '1';
				busy <= '1';
				sr(14 downto 5) <= "00" & TTS_fifo_Do_q & "000" & SendBC0;
				sr(4) <= TTS_fifo_Do_q(3) xor TTS_fifo_Do_q(2) xor TTS_fifo_Do_q(1) xor SendBC0; 
				sr(3) <= TTS_fifo_Do_q(3) xor TTS_fifo_Do_q(2) xor TTS_fifo_Do_q(0); 
				sr(2) <= TTS_fifo_Do_q(3) xor TTS_fifo_Do_q(1) xor TTS_fifo_Do_q(0); 
				sr(1) <= TTS_fifo_Do_q(2) xor TTS_fifo_Do_q(1) xor TTS_fifo_Do_q(0) xor SendBC0; 
				sr(0) <= SendBC0; 
			else
				start <= '0';
				if(stop = '1')then
					busy <= '0';
				end if;
				sr <= sr(13 downto 0) & '1';
			end if;
		end if;
		if(TTC_strobe = '1')then	
			phase <= "00";
		else
			phase(1) <= phase(1) xor phase(0);
			phase(0) <= not phase(0);
		end if;
		if(local_TTC = '1')then
			phase_l(1) <= phase_l(1) xor phase_l(0);
			phase_l(0) <= not phase_l(0);
			TTS_out <= TTS_out xor (phase_l(1) or phase_l(0));
		else
			case phase is
				when "01" => TTS_out <= TTS_out xor L1A_trig;
				when "11" => TTS_out <= TTS_out xor sr(14);
				when others => TTS_out <= not TTS_out;
			end case;
		end if;
		TTS_out_q <= TTS_out;
	end if;
end process;
i_SendBC0 : SRL16E
   port map (
      Q => SendBC0,       -- SRL data output
      A0 => '1',     -- Select[0] input
      A1 => '1',     -- Select[1] input
      A2 => '1',     -- Select[2] input
      A3 => '1',     -- Select[3] input
      CE => TTC_strobe,     -- Clock enable input
      CLK => CDRclk,   -- Clock input
      D => inh_msg        -- SRL data input
   );
i_stop : SRL16E
   port map (
      Q => stop,       -- SRL data output
      A0 => '1',     -- Select[0] input
      A1 => '1',     -- Select[1] input
      A2 => '1',     -- Select[2] input
      A3 => '1',     -- Select[3] input
      CE => TTC_strobe,     -- Clock enable input
      CLK => CDRclk,   -- Clock input
      D => start        -- SRL data input
   );

end Behavioral;

