----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:02:29 01/25/2013 
-- Design Name: 
-- Module Name:    ddr_wportA - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.std_logic_misc.all;
use work.amc13_pack.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;
Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity ddr_rport is
    Port ( memclk : in  STD_LOGIC;
           sysclk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           resetSys : in  STD_LOGIC;
           resetMem : in  STD_LOGIC;
           run : in  STD_LOGIC;
           test : in  STD_LOGIC_VECTOR(1 downto 0); -- memory test
					 test_block_sent	: in  STD_LOGIC; -- indicating test writing progress. One pulse every 256 32-bit words
					 test_pause : OUT std_logic;
					 test_status	: out  STD_LOGIC_VECTOR(63 downto 0);
-- each TCP segment sent will have dest IP address (32-bit), TCP port(2x16-bit), SEQ number(32-bit), length(16 bit), time stamp(16 bit), control bits(6)
-- and memory starting address of the segment payload data(24-bit)
-- for alignment purposes, each entry occupies 32 bytes of memory space (256 bits)
           TCP_dout : out  STD_LOGIC_VECTOR(31 downto 0); -- TCP data are written in unit of 32-bit words
           TCP_addr : in  array3X24;
           TCP_length : in  array3X14; -- in 32 bit word
           TCP_dout_valid : out  STD_LOGIC_VECTOR (2 downto 0);
           TCP_rqst : in  STD_LOGIC_VECTOR (2 downto 0);
           TCP_ack : out  STD_LOGIC_VECTOR (2 downto 0);
					 TCP_lastword : out  STD_LOGIC;
--	ipbus signals
					 ipb_clk : in  STD_LOGIC;
					 ipb_write : in  STD_LOGIC;
					 ipb_strobe	: in  STD_LOGIC;
					 ipb_addr	: in  STD_LOGIC_VECTOR(31 downto 0);
					 ipb_rdata : out  STD_LOGIC_VECTOR(31 downto 0);
					 ipb_ack : out  STD_LOGIC;
					 page_addr	: in  STD_LOGIC_VECTOR(11 downto 0); -- monitored event memory window
-- ddr3 user interface
           app_rqst : out  STD_LOGIC; -- request to output data
           app_ack : in  STD_LOGIC; -- permission to output data
           app_en_accept : in  STD_LOGIC; -- permission to output data
           app_en : out  STD_LOGIC;
           app_rd_data_valid : in  STD_LOGIC;
					 app_rd_data : in  STD_LOGIC_VECTOR(255 downto 0);
           app_addr : out  STD_LOGIC_VECTOR (27 downto 0) := (others => '0')); -- starting address of write burst(1Kbytes or until end of event)
end ddr_rport;

architecture Behavioral of ddr_rport is
signal TCP_ack_i : std_logic_vector(2 downto 0) := (others => '0');
signal ipb_rbuf_wa : std_logic_vector(4 downto 0) := (others => '0');
signal ipb_rbuf_wap : std_logic_vector(1 downto 0) := (others => '0');
signal ipb_rbuf_ra : std_logic_vector(4 downto 0) := (others => '0');
signal ipb_rbuf_do : std_logic_vector(31 downto 0) := (others => '0');
signal ipb_start_addr : std_logic_vector(26 downto 0) := (others => '0');
signal ipb_rbuf_wa4SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal ipb_rbuf_wa3SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal ipbReadSyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal ipb_rbuf_we : std_logic := '0';
signal ipbRead : std_logic := '0';
signal ipbRead_q : std_logic := '0';
signal ipb_rbuf_rdy : std_logic := '0';
signal ipb_ack_i : std_logic := '0';
signal ipb_strobe_q : std_logic := '0';
signal ipb_rqst : std_logic := '0';
signal ipb_wc : std_logic_vector(7 downto 0) := (others => '0');
signal ipb_raddr : std_logic_vector(23 downto 0) := (others => '0');
signal ipb_seq : std_logic_vector(1 downto 0) := (others => '0');
signal ipb_rqst_cnt : std_logic_vector(1 downto 0) := (others => '0');
signal ipb_rqstSyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal queue_we : std_logic := '0';
signal queue_do_vld : std_logic := '0';
signal queue_di : std_logic_vector(28 downto 0) := (others => '0');
signal queue_dop : std_logic_vector(28 downto 0) := (others => '0');
signal queue_do : std_logic_vector(28 downto 0) := (others => '0');
signal queue_wa : std_logic_vector(3 downto 0) := (others => '0');
signal queue_ra : std_logic_vector(3 downto 0) := (others => '0');
signal queue_wap : std_logic_vector(3 downto 0) := (others => '0');
signal queue_wa0SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal queue_wa1SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal queue_rap : std_logic_vector(3 downto 0) := (others => '0');
signal queue_ra0SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal queue_ra1SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal Oqueue_di : std_logic_vector(11 downto 0) := (others => '0');
signal Oqueue_dop : std_logic_vector(11 downto 0) := (others => '0');
signal Oqueue_do : std_logic_vector(11 downto 0) := (others => '0');
signal Oqueue_a : std_logic_vector(3 downto 0) := (others => '1');
signal queue_full : std_logic := '0';
signal Oqueue_re : std_logic := '0';
signal test_wc : std_logic_vector(5 downto 0) := (others => '0');
signal testSyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal lfsr: std_logic_vector(31 downto 0) := (others => '0');
signal testErr : std_logic := '0';
signal testCntr: std_logic_vector(30 downto 0) := (others => '0');
signal app_rqst_i : std_logic := '0';
signal s_demux : std_logic_vector(2 downto 0) := (others => '0');
signal data_demux : std_logic_vector(31 downto 0) := (others => '0');
signal lengthCntr : std_logic_vector(7 downto 0) := (others => '0');
signal wc : std_logic_vector(4 downto 0) := (others => '0');
signal app_addr_i : std_logic_vector(27 downto 0) := (others => '0');
signal ec_buf_do : std_logic := '0';
signal buf_do_vld : std_logic := '0';
signal buf_do_vld_q : std_logic := '0';
signal ec_buf_ra : std_logic := '0';
signal buf_wa : std_logic_vector(8 downto 0) := (others => '0');
signal buf_wap : std_logic_vector(5 downto 0) := (others => '0');
signal buf_wa0SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal buf_wa1SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal buf_ra : std_logic_vector(8 downto 0) := (others => '0');
signal buf_do : std_logic_vector(255 downto 0) := (others => '0');
signal TCP_rrqst : std_logic := '0';
signal TCP_raddr : std_logic_vector(23 downto 0) := (others => '0');
signal TCP_rlengthCntr : std_logic_vector(13 downto 0) := (others => '0');
signal TCP_rlengthp : std_logic_vector(7 downto 0) := (others => '0');
signal TCP_rlength : std_logic_vector(7 downto 0) := (others => '0');
signal TCP_channel : std_logic_vector(1 downto 0) := (others => '0');
signal TCPqueue_di : std_logic_vector(39 downto 0) := (others => '0');
signal TCPqueue_do : std_logic_vector(39 downto 0) := (others => '0');
signal TCPqueue_a : std_logic_vector(3 downto 0) := (others => '1');
signal TCP_pause : std_logic := '0';
signal TCPqueue_we : std_logic := '0';
signal en_test_read : std_logic := '0';
begin
test_pause <= test_wc(5);
ipb_ack <= ipb_ack_i;
app_rqst <= app_rqst_i;
app_en <= app_ack and app_rqst_i;
TCP_ack <= TCP_ack_i;
test_status(31 downto 0) <= testErr & testCntr;
test_status(47 downto 32) <= lfsr(15 downto 0);
test_status(57 downto 48) <= (others => '0');
test_status(63 downto 58) <= test_wc;
app_addr <= app_addr_i;
TCP_dout <= data_demux;
process(ipb_clk)
begin
	if(ipb_clk'event and ipb_clk = '1')then
		ipb_strobe_q <= ipb_strobe;
		if(reset = '1' or ipb_rqst = '1' or ipbRead = '0')then
			ipb_rqst <= '0';
		elsif(ipb_rbuf_rdy = '0' or (ipb_strobe = '1' and ipb_strobe_q = '1' and ipb_rqst_cnt /= "11"))then
			ipb_rqst <= '1';
		end if;
		if(ipbRead = '0')then
			ipb_rqst_cnt <= "00";
		elsif(ipb_rqst = '1')then
			ipb_rqst_cnt <= ipb_rqst_cnt + 1;
		elsif(ipb_ack_i = '1' and ipb_rbuf_ra(2 downto 0) = "000")then
			ipb_rqst_cnt <= ipb_rqst_cnt - 1;
		end if;
		if(reset = '1' or (ipb_wc = x"01" and ipb_ack_i = '1'))then
			ipb_rbuf_rdy <= '0';
		elsif(ipb_rqst = '1')then
			ipb_rbuf_rdy <= '1';
		end if;
		if(reset = '1' or ipbRead = '0')then
			ipb_wc <= (others => '0');
		elsif(ipb_strobe_q = '1' and ipb_ack_i = '0')then
			ipb_wc <= ipb_wc + 1;
		elsif(ipb_strobe_q = '0' and ipb_ack_i = '1')then
			ipb_wc <= ipb_wc - 1;
		end if;
		if(reset = '1' or test(0) = '1' or ipb_write = '1' or (ipb_addr(27) = '0' and ipb_addr(14) = '0'))then
			ipbRead <= '0';
		elsif(ipb_strobe = '1' and ipb_strobe_q = '0')then
			ipbRead <= '1';
		elsif(ipb_wc = x"01" and ipb_ack_i = '1')then
			ipbRead <= '0';
		end if;
		ipbRead_q <= ipbRead;
		if(ipb_write = '0' and ipb_strobe = '1' and ipb_strobe_q = '0')then
			if(ipb_addr(27) = '1')then
				ipb_start_addr <= ipb_addr(26 downto 0);
			elsif(ipb_addr(14) = '1')then
				ipb_start_addr <= '0' & page_addr & ipb_addr(13 downto 0);
			end if;
		end if;
		if(ipb_rbuf_rdy = '0' or ipb_rbuf_wap = ipb_rbuf_ra(4 downto 3) or (ipb_wc = x"01" and ipb_ack_i = '1'))then
			ipb_ack_i <= '0';
		else
			ipb_ack_i <= '1';
		end if;
		if(ipbRead = '0')then
			ipb_rbuf_ra <= (others => '0');
		elsif(ipbRead_q = '0')then
			ipb_rbuf_ra(4) <= ipb_start_addr(4);
			ipb_rbuf_ra(3) <= ipb_start_addr(4) xor ipb_start_addr(3);
			ipb_rbuf_ra(2 downto 0) <= ipb_start_addr(2 downto 0);
		elsif(ipb_rbuf_rdy = '1' and ipb_rbuf_wap /= ipb_rbuf_ra(4 downto 3))then
			ipb_rbuf_ra(2 downto 0) <= ipb_rbuf_ra(2 downto 0) + 1;
			if(ipb_rbuf_ra(2 downto 0) = "111")then
				ipb_rbuf_ra(4) <= ipb_rbuf_ra(3);
				ipb_rbuf_ra(3) <= not ipb_rbuf_ra(4);
			end if;
		end if;
		if(ipbRead = '0')then
			ipb_rbuf_wap <= (others => '0');
		elsif(ipbRead_q = '0')then
			ipb_rbuf_wa4SyncRegs <= ipb_start_addr(4) & ipb_start_addr(4) & ipb_start_addr(4);
			ipb_rbuf_wa3SyncRegs <= (ipb_start_addr(4) xor ipb_start_addr(3)) & (ipb_start_addr(4) xor ipb_start_addr(3)) & (ipb_start_addr(4) xor ipb_start_addr(3));
			ipb_rbuf_wap <= ipb_start_addr(4 downto 3);
		else
			ipb_rbuf_wa4SyncRegs <= ipb_rbuf_wa4SyncRegs(1 downto 0) & ipb_rbuf_wa(4);
			ipb_rbuf_wa3SyncRegs <= ipb_rbuf_wa3SyncRegs(1 downto 0) & ipb_rbuf_wa(3);
			ipb_rbuf_wap(1) <= ipb_rbuf_wa4SyncRegs(2);
			ipb_rbuf_wap(0) <= ipb_rbuf_wa3SyncRegs(2);
		end if;
		ipb_rdata <= ipb_rbuf_do;
--		ipb_rdata <= ipb_rbuf(conv_integer(ipb_rbuf_ra(4 downto 0)));
	end if;
end process;
g_ipb_rbuf : for i in 0 to 31 generate
   i_ipb_rbuf : RAM32X1D
   port map (
      DPO => ipb_rbuf_do(i),     -- Read-only 1-bit data output
      SPO => open,     -- R/W 1-bit data output
      A0 => ipb_rbuf_wa(0),       -- R/W address[0] input bit
      A1 => ipb_rbuf_wa(1),       -- R/W address[1] input bit
      A2 => ipb_rbuf_wa(2),       -- R/W address[2] input bit
      A3 => ipb_rbuf_wa(3),       -- R/W address[3] input bit
      A4 => ipb_rbuf_wa(4),       -- R/W address[4] input bit
      D => data_demux(i),         -- Write 1-bit data input
      DPRA0 => ipb_rbuf_ra(0), -- Read-only address[0] input bit
      DPRA1 => ipb_rbuf_ra(1), -- Read-only address[1] input bit
      DPRA2 => ipb_rbuf_ra(2), -- Read-only address[2] input bit
      DPRA3 => ipb_rbuf_ra(3), -- Read-only address[3] input bit
      DPRA4 => ipb_rbuf_ra(4), -- Read-only address[4] input bit
      WCLK => sysclk,   -- Write clock input
      WE => ipb_rbuf_we       -- Write enable input
   );
end generate;
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		testSyncRegs <= testSyncRegs(1 downto 0) & test(0);
		ipbReadSyncRegs <= ipbReadSyncRegs(2 downto 0) & ipbRead;
		ipb_rqstSyncRegs <= ipb_rqstSyncRegs(2 downto 0) & ipb_rqst;
		if(ipbReadSyncRegs(3 downto 2) = "01")then
			ipb_rbuf_wa(4) <= ipb_start_addr(4);
			ipb_rbuf_wa(3) <= ipb_start_addr(4) xor ipb_start_addr(3);
			ipb_rbuf_wa(2 downto 0) <= "000";
		elsif(ipb_rbuf_we = '1')then
			ipb_rbuf_wa(2 downto 0) <= ipb_rbuf_wa(2 downto 0) + 1;
			if(ipb_rbuf_wa(2 downto 0) = "111")then
				ipb_rbuf_wa(4) <= ipb_rbuf_wa(3);
				ipb_rbuf_wa(3) <= not ipb_rbuf_wa(4);
			end if;
		end if;
		if(resetSys = '1' or testSyncRegs(2) = '1')then
			ipb_rbuf_we <= '0';
		elsif(buf_do_vld = '1' and Oqueue_do(1 downto 0) = "11" and Oqueue_do(3 downto 2) = ipb_seq)then
			ipb_rbuf_we <= '1';
		else
			ipb_rbuf_we <= '0';
		end if;
	end if;
end process;
g_TCPqueue: for i in 0 to 39 generate
	i_TCPqueue : SRL16E
   port map (
      Q => TCPqueue_do(i),       -- SRL data output
      A0 => TCPqueue_a(0),     -- Select[0] input
      A1 => TCPqueue_a(1),     -- Select[1] input
      A2 => TCPqueue_a(2),     -- Select[2] input
      A3 => TCPqueue_a(3),     -- Select[3] input
      CE => TCPqueue_we,     -- Clock enable input
      CLK => sysclk,   -- Clock input
      D => TCPqueue_di(i)        -- SRL data input
   );
end generate;
g_Oqueue: for i in 0 to 11 generate
	i_Oqueue : SRL16E
   port map (
      Q => Oqueue_dop(i),       -- SRL data output
      A0 => Oqueue_a(0),     -- Select[0] input
      A1 => Oqueue_a(1),     -- Select[1] input
      A2 => Oqueue_a(2),     -- Select[2] input
      A3 => Oqueue_a(3),     -- Select[3] input
      CE => queue_we,     -- Clock enable input
      CLK => sysclk,   -- Clock input
      D => Oqueue_di(i)        -- SRL data input
   );
end generate;
g_queue : for i in 0 to 28 generate
   i_queue : RAM32X1D
   port map (
      DPO => queue_dop(i),     -- Read-only 1-bit data output
      SPO => open,     -- R/W 1-bit data output
      A0 => queue_wa(0),       -- R/W address[0] input bit
      A1 => queue_wa(1),       -- R/W address[1] input bit
      A2 => queue_wa(2),       -- R/W address[2] input bit
      A3 => queue_wa(3),       -- R/W address[3] input bit
      A4 => '0',       -- R/W address[4] input bit
      D => queue_di(i),         -- Write 1-bit data input
      DPRA0 => queue_ra(0), -- Read-only address[0] input bit
      DPRA1 => queue_ra(1), -- Read-only address[1] input bit
      DPRA2 => queue_ra(2), -- Read-only address[2] input bit
      DPRA3 => queue_ra(3), -- Read-only address[3] input bit
      DPRA4 => '0', -- Read-only address[4] input bit
      WCLK => sysclk,   -- Write clock input
      WE => queue_we       -- Write enable input
   );
end generate;
TCP_rlengthp <= TCP_rlength + x"07";
queue_full <= '1' when Oqueue_a = x"b" or Oqueue_a = x"c" or Oqueue_a = x"d" or Oqueue_a = x"e" else '0';
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		if(resetSys = '1' or run = '0' or testSyncRegs(2) = '1' or TCPqueue_a = x"d" or TCPqueue_a = x"e")then
			TCP_ack_i <= "000";
		elsif(TCP_ack_i(0) = '1')then
			TCP_ack_i(0) <= '0';
			if(TCP_rqst(1) = '1')then
				TCP_ack_i(1) <= '1';
			elsif(TCP_rqst(2) = '1')then
				TCP_ack_i(2) <= '1';
			end if;
		elsif(TCP_ack_i(1) = '1')then
			TCP_ack_i(1) <= '0';
			if(TCP_rqst(2) = '1')then
				TCP_ack_i(2) <= '1';
			elsif(TCP_rqst(0) = '1')then
				TCP_ack_i(0) <= '1';
			end if;
		elsif(TCP_ack_i(2) = '1')then
			TCP_ack_i(2) <= '0';
			if(TCP_rqst(0) = '1')then
				TCP_ack_i(0) <= '1';
			elsif(TCP_rqst(1) = '1')then
				TCP_ack_i(1) <= '1';
			end if;
		else
			if(TCP_rqst(0) = '1')then
				TCP_ack_i(0) <= '1';
			elsif(TCP_rqst(1) = '1')then
				TCP_ack_i(1) <= '1';
			elsif(TCP_rqst(2) = '1')then
				TCP_ack_i(2) <= '1';
			end if;
		end if;
		if(TCP_ack_i(2) = '1')then
			TCPqueue_di <= "10" & TCP_length(2) & TCP_addr(2);
		elsif(TCP_ack_i(1) = '1')then
			TCPqueue_di <= "01" & TCP_length(1) & TCP_addr(1);
		else
			TCPqueue_di <= "00" & TCP_length(0) & TCP_addr(0);
		end if;
		if(resetSys = '1' or TCP_ack_i = "000")then
			TCPqueue_we <= '0';
		else
			TCPqueue_we <= '1';
		end if;
		if(resetSys = '1')then
			TCPqueue_a <= (others => '1');
		elsif(TCPqueue_we = '1' and (and_reduce(TCPqueue_a) = '1' or TCP_rrqst = '1'))then
			TCPqueue_a <= TCPqueue_a + 1;
		elsif(TCPqueue_we = '0' and and_reduce(TCPqueue_a) = '0' and TCP_rrqst = '0')then
			TCPqueue_a <= TCPqueue_a - 1;
		end if;
		if(resetSys = '1')then
			TCP_rrqst <= '0';
		elsif(queue_we = '1' and Oqueue_di(11) = '1')then
			TCP_rrqst <= '0';
		elsif(and_reduce(TCPqueue_a) = '0')then
			TCP_rrqst <= '1';
		end if;
		if(TCP_rrqst = '0')then
			TCP_channel <= TCPqueue_do(39 downto 38);
			TCP_raddr <= TCPqueue_do(23 downto 0);
			TCP_rlengthCntr <= TCPqueue_do(37 downto 24);
			if(or_reduce(TCPqueue_do(37 downto 32)) = '1')then
				TCP_rlength <= x"00";
			else
				TCP_rlength <= TCPqueue_do(31 downto 24);
			end if;
		elsif(queue_we = '1' and Oqueue_di(1 downto 0) /= "11")then
			TCP_raddr(23 downto 5) <= TCP_raddr(23 downto 5) + 1;
			TCP_rlengthCntr(13 downto 8) <= TCP_rlengthCntr(13 downto 8) - 1;
			if(or_reduce(TCP_rlengthCntr(13 downto 9)) = '1')then
				TCP_rlength <= x"00";
			else
				TCP_rlength <= TCP_rlengthCntr(7 downto 0);
			end if;
		end if;
		if(resetSys = '1')then
			TCP_pause <= '0';
		elsif(Oqueue_re = '1' and Oqueue_do(10) = '1')then
			TCP_pause <= '0';
		elsif(queue_we = '1' and Oqueue_di(10) = '1')then
			TCP_pause <= '1';
		end if;
		if(resetSys = '1' or Oqueue_do(1 downto 0) = "11" or buf_do_vld = '0' or Oqueue_re = '1')then
			TCP_dout_valid <= "000";
		else
			TCP_dout_valid(conv_integer(Oqueue_do(1 downto 0))) <= '1';
		end if;
		if(ipbReadSyncRegs(3 downto 2) = "10")then -- ipb_seq is used to eject stalemate read data
			ipb_seq <= ipb_seq + 1;
		end if;
		if(resetSys = '1')then
			ipb_raddr <= (others => '0');
		elsif(ipbReadSyncRegs(3 downto 2) = "01")then
			ipb_raddr <= ipb_start_addr(26 downto 3);
		elsif(queue_we = '1' and Oqueue_di(1 downto 0) = "11")then
			ipb_raddr <= ipb_raddr + 1; 
		end if;
		case s_demux is
			when "000" => data_demux <= buf_do(31 downto 0);
			when "001" => data_demux <= buf_do(63 downto 32);
			when "010" => data_demux <= buf_do(95 downto 64);
			when "011" => data_demux <= buf_do(127 downto 96);
			when "100" => data_demux <= buf_do(159 downto 128);
			when "101" => data_demux <= buf_do(191 downto 160);
			when "110" => data_demux <= buf_do(223 downto 192);
			when others => data_demux <= buf_do(255 downto 224);
		end case;
		if(buf_do_vld = '0' or (Oqueue_re = '1' and Oqueue_do(11) = '1'))then
			s_demux <= "000";
		else
			s_demux <= s_demux + 1;
		end if;
		if(resetSys = '1' or testSyncRegs(2) = '0')then
			en_test_read <= '0';
		elsif(test_block_sent = '1')then
			en_test_read <= '1';
		end if;
		if(resetSys = '1' or testSyncRegs(2) = '0' or en_test_read = '0')then
			test_wc <= (others => '0');
		elsif(test_block_sent = '1' and queue_we = '0')then
			test_wc <= test_wc + 1;
		elsif(test_block_sent = '0' and queue_we = '1')then
			test_wc <= test_wc - 1;
		end if;
		if(resetSys = '1' or queue_we = '1')then
			queue_we <= '0';
		elsif((or_reduce(test_wc(5 downto 1)) = '1' and queue_full = '0') or ipb_rqstSyncRegs(3 downto 2) = "10" or (TCP_rrqst = '1' and queue_full = '0' and TCP_pause = '0'))then
			queue_we <= '1';
		else
			queue_we <= '0';
		end if;
		if(testSyncRegs(2) = '1' or ipb_rqstSyncRegs(3 downto 2) = "10")then
			queue_di(23 downto 0) <= ipb_raddr;
			queue_di(28 downto 24) <= "00001";
			Oqueue_di(1 downto 0) <= "11";
			Oqueue_di(10 downto 2) <= "0000000" & ipb_seq;
		else
			queue_di(23 downto 0) <= TCP_raddr;
			queue_di(28 downto 24) <= TCP_rlengthp(7 downto 3);
			Oqueue_di(1 downto 0) <= TCP_channel;
			Oqueue_di(9 downto 2) <= TCP_rlength;
			if(TCP_rlengthp(7 downto 3) = "00001")then
				Oqueue_di(10) <= '0';
			else
				Oqueue_di(10) <= '1';
			end if;
			if(TCP_rlength = x"00")then
				Oqueue_di(11) <= '0';
			else
				Oqueue_di(11) <= '1';
			end if;
		end if;
		Oqueue_do <= Oqueue_dop;
		if(resetSys = '1' or Oqueue_re = '1')then
			lengthCntr <= x"01";
		elsif(buf_do_vld = '1')then
			lengthCntr <= lengthCntr + 1;
		end if;
		if(resetSys = '1')then
			Oqueue_re <= '0';
			TCP_lastword <= '0';
		elsif((lengthCntr = Oqueue_do(9 downto 2) and Oqueue_do(1 downto 0) /= "11") or (Oqueue_do(1 downto 0) = "11" and s_demux = "111"))then
			Oqueue_re <= '1';
			TCP_lastword <= Oqueue_do(11);
		else
			Oqueue_re <= '0';
			TCP_lastword <= '0';
		end if;
		if(resetSys = '1')then
			Oqueue_a <= (others => '1');
		elsif(queue_we = '1' and Oqueue_re = '0')then
			Oqueue_a <= Oqueue_a + 1;
		elsif(queue_we = '0' and Oqueue_re = '1')then
			Oqueue_a <= Oqueue_a - 1;
		end if;
		if(resetSys = '1')then
			queue_wa <= (others => '0');
		elsif(queue_we = '1')then
			queue_wa(1) <= queue_wa(0);
			queue_wa(0) <= not queue_wa(1);
			if(queue_wa(1 downto 0) = "10")then
				queue_wa(3 downto 2) <= queue_wa(3 downto 2) + 1;
			end if;
		end if;
		if(resetSys = '1')then
			testErr <= '0';
			testCntr <= (others => '0');
		elsif(testSyncRegs(2) = '1' and buf_do_vld_q = '1')then
			if(lfsr /= data_demux)then
				testErr <= '1';
			end if;
			if(testErr = '0')then
				testCntr <= testCntr + 1;
			end if;
		end if;
		buf_do_vld_q <= buf_do_vld;
		if(testSyncRegs(2) = '0')then
			lfsr <= (others => '0');
		elsif(buf_do_vld_q = '1')then
			if(test(1) = '0')then
				lfsr <= lfsr(30 downto 0) & not(lfsr(31) xor lfsr(21) xor lfsr(1) xor lfsr(0));
			else
				lfsr <= lfsr + 1;
			end if;
		end if;
		if(resetSys = '1')then
			queue_ra0SyncRegs <= (others => '0');
			queue_ra1SyncRegs <= (others => '0');
			queue_rap <= (others => '0');
		else
			queue_ra0SyncRegs <= queue_ra0SyncRegs(1 downto 0) & queue_ra(0);
			queue_ra1SyncRegs <= queue_ra1SyncRegs(1 downto 0) & queue_ra(1);
			queue_rap(1) <= queue_ra1SyncRegs(2);
			queue_rap(0) <= queue_ra0SyncRegs(2);
			if(queue_rap(1) = '1' and queue_ra1SyncRegs(2) = '0')then
				queue_rap(3 downto 2) <= queue_rap(3 downto 2) + 1;
			end if;
		end if;
		if(resetSys = '1')then
			buf_wa0SyncRegs <= (others => '0');
			buf_wa1SyncRegs <= (others => '0');
			buf_wap <= (others => '0');
		else
			buf_wa0SyncRegs <= buf_wa0SyncRegs(1 downto 0) & buf_wa(0);
			buf_wa1SyncRegs <= buf_wa1SyncRegs(1 downto 0) & buf_wa(1);
			buf_wap(1) <= buf_wa1SyncRegs(2);
			buf_wap(0) <= buf_wa0SyncRegs(2);
			if(buf_wap(1) = '1' and buf_wa1SyncRegs(2) = '0')then
				buf_wap(5 downto 2) <= buf_wap(5 downto 2) + 1;
			end if;
		end if;
		if(resetSys = '1' or ec_buf_ra = '1')then
			ec_buf_ra <= '0';
		elsif((buf_do_vld = '0' or s_demux = "110") and buf_ra(5 downto 0) /= buf_wap)then
			ec_buf_ra <= '1';
		end if;
		if(resetSys = '1' or (Oqueue_re = '1' and Oqueue_do(11) = '1'))then
			buf_do_vld <= '0';
		elsif(ec_buf_do = '1')then
			buf_do_vld <= '1';
		elsif(s_demux = "111")then
			buf_do_vld <= '0';
		end if;
		if(resetSys = '1')then
			buf_ra <= (others => '0');
		elsif(ec_buf_ra = '1')then
			buf_ra(1) <= buf_ra(0);
			buf_ra(0) <= not buf_ra(1);
			if(buf_ra(1 downto 0) = "10")then
				buf_ra(5 downto 2) <= buf_ra(5 downto 2) + 1;
			end if;
		end if;
	end if;
end process;
process(memclk)
begin
	if(memclk'event and memclk = '1')then
		if((queue_wap /= queue_ra and queue_do_vld = '0') or (wc = "00001" and app_en_accept = '1'))then
			queue_do <= queue_dop;
		end if;
		if(resetMem = '1')then
			queue_do_vld <= '0';
		elsif(queue_wap /= queue_ra)then
			queue_do_vld <= '1';
		elsif(wc = "00001" and app_en_accept = '1')then
			queue_do_vld <= '0';
		end if;
		if(resetMem = '1')then
			queue_ra <= (others => '0');
		elsif(queue_wap /= queue_ra and (queue_do_vld = '0' or (wc = "00001" and app_en_accept = '1')))then
			queue_ra(1) <= queue_ra(0);
			queue_ra(0) <= not queue_ra(1);
			if(queue_ra(1 downto 0) = "10")then
				queue_ra(3 downto 2) <= queue_ra(3 downto 2) + 1;
			end if;
		end if;
		if(resetMem = '1')then
			queue_wa0SyncRegs <= (others => '0');
			queue_wa1SyncRegs <= (others => '0');
			queue_wap <= (others => '0');
		else
			queue_wa0SyncRegs <= queue_wa0SyncRegs(1 downto 0) & queue_wa(0);
			queue_wa1SyncRegs <= queue_wa1SyncRegs(1 downto 0) & queue_wa(1);
			queue_wap(1) <= queue_wa1SyncRegs(2);
			queue_wap(0) <= queue_wa0SyncRegs(2);
			if(queue_wap(1) = '1' and queue_wa1SyncRegs(2) = '0')then
				queue_wap(3 downto 2) <= queue_wap(3 downto 2) + 1;
			end if;
		end if;
		if(resetMem = '1' or (wc = "00001" and app_en_accept = '1'))then
			app_rqst_i <= '0';
		elsif(queue_do_vld = '1')then
			app_rqst_i <= '1';
		end if;
		if(app_rqst_i = '0')then
			app_addr_i <= '0' & queue_do(23 downto 0) & "000";
			wc <= queue_do(28 downto 24);
		elsif(app_en_accept = '1')then
			app_addr_i(26 downto 3) <= app_addr_i(26 downto 3) + 1;
			wc <= wc - 1;
		end if;
		if(resetMem = '1')then
			buf_wa <= (others => '0');
		elsif(app_rd_data_valid = '1')then
			buf_wa(1) <= buf_wa(0);
			buf_wa(0) <= not buf_wa(1);
			if(buf_wa(1 downto 0) = "10")then
				buf_wa(5 downto 2) <= buf_wa(5 downto 2) + 1;
			end if;
		end if;
	end if;
end process;
g_rbuf : for i in 0 to 3 generate
	i_rbuf : BRAM_SDP_MACRO
   generic map (
      BRAM_SIZE => "36Kb", -- Target BRAM, "18Kb" or "36Kb" 
      DEVICE => "7SERIES", -- Target device: "VIRTEX5", "VIRTEX6", "7SERIES", "SPARTAN6" 
      WRITE_WIDTH => 64,    -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      READ_WIDTH => 64,     -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      DO_REG => 1, -- Optional output register (0 or 1)
      SIM_COLLISION_CHECK => "NONE", -- Collision check enable "ALL", "WARNING_ONLY", 
                                    -- "GENERATE_X_ONLY" or "NONE"       
      WRITE_MODE => "WRITE_FIRST", -- Specify "READ_FIRST" for same clock or synchronous clocks
                                   --  Specify "WRITE_FIRST for asynchrononous clocks on ports
      INIT => X"000000000000000000") --  Initial values on output port
   port map (
      DO => buf_do(i*64+63 downto i*64),         -- Output read data port, width defined by READ_WIDTH parameter
      DI => app_rd_data(i*64+63 downto i*64),         -- Input write data port, width defined by WRITE_WIDTH parameter
      RDADDR => buf_ra, -- Input read address, width defined by read port depth
      RDCLK => sysclk,   -- 1-bit input read clock
      RDEN => '1',     -- 1-bit input read port enable
      REGCE => ec_buf_do,   -- 1-bit input read output register enable
      RST => '0',       -- 1-bit input reset 
      WE => x"ff",         -- Input write enable, width defined by write port depth
      WRADDR => buf_wa, -- Input write address, width defined by write port depth
      WRCLK => memclk,   -- 1-bit input write clock
      WREN => app_rd_data_valid      -- 1-bit input write port enable
   );
end generate;
ec_buf_do <= '1' when resetSys = '0' and ec_buf_ra = '1' and (buf_do_vld = '0' or s_demux = "111") else '0';
end Behavioral;

