----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:02:29 01/25/2013 
-- Design Name: 
-- Module Name:    ddr_wportA - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.std_logic_misc.all;
use work.amc13_pack.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;
Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity ddr_wportA is
    Port ( sysclk : in  STD_LOGIC;
           memclk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           resetSys : in  STD_LOGIC;
           resetMem : in  STD_LOGIC;
           run : in  STD_LOGIC;
           din : in  STD_LOGIC_VECTOR (64 downto 0);
           din_we : in  STD_LOGIC;
           port_rdy : out  STD_LOGIC; -- port ready to accept a new event
           buf_full : out  STD_LOGIC;
           event_addr : in  STD_LOGIC_VECTOR (12 downto 0);
           addr_we : in  STD_LOGIC;
           rqst : out  STD_LOGIC; -- request to output data
           ack : in  STD_LOGIC; -- permission to output data
           app_wdf_rdy : in  STD_LOGIC;
           app_rdy : in  STD_LOGIC;
           app_en : out  STD_LOGIC;
           app_wdf_wren : out  STD_LOGIC;
           app_addr : out  STD_LOGIC_VECTOR (23 downto 0) := (others => '0'); -- starting address of write burst(1Kbytes or until end of event)
           dout : out  STD_LOGIC_VECTOR (255 downto 0) := (others => '0'));
end ddr_wportA;

architecture Behavioral of ddr_wportA is
signal en_din : std_logic := '0';
signal buf_do_vld : std_logic := '0';
signal dout_vld : std_logic := '0';
signal rqst_i : std_logic := '0';
signal got_eoe : std_logic := '0';
signal got_eoeSyncRegs : std_logic_vector(4 downto 0) := (others => '0');
signal EventDone : std_logic := '0';
signal EventDataDone : std_logic := '0';
signal buf_we : std_logic_vector(3 downto 0) := (others => '0');
signal buf_re : std_logic := '0';
signal buf_wa : std_logic_vector(12 downto 0) := (others => '0');
signal buf_ra2SyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal buf_rap : std_logic_vector(8 downto 0) := (others => '0');
signal buf_ra : std_logic_vector(10 downto 0) := (others => '0');
signal buf_wa2SyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal buf_wap : std_logic_vector(10 downto 0) := (others => '0');
signal buf_di : std_logic_vector(71 downto 0) := (others => '0');
signal buf_do : std_logic_vector(287 downto 0) := (others => '0');
signal dout_wc : std_logic_vector(4 downto 0) := (others => '0');
signal wc_left : std_logic_vector(5 downto 0) := (others => '0');
signal en_app_en : std_logic := '0';
signal bufRdEn : std_logic := '0';
signal bufREGCE : std_logic := '0';
signal app_wdf_accept : std_logic := '0';
signal app_addr_i : std_logic_vector(23 downto 0) := (others => '0');

begin
rqst <= rqst_i or en_app_en;
app_wdf_wren <= ack and rqst_i;
app_en <= ack and en_app_en;
app_addr <= app_addr_i;
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		if(addr_we = '1')then
			app_addr_i(23 downto 11) <= event_addr;
		end if;
		port_rdy <= not en_din;
	end if;
end process;
process(sysclk,EventDone)
begin
	if(EventDone = '1')then
		en_din <= '0';
		buf_wa <= (others => '0');
		got_eoe <= '0';
		buf_ra2SyncRegs <= (others => '0');
		buf_rap <= (others => '0');
		buf_full <= '0';
	elsif(sysclk'event and sysclk = '1')then
		if(resetSys = '1')then
			en_din <= '0';
		elsif(addr_we = '1')then
			en_din <= '1';
		end if;
		if(reset = '1' or en_din = '0')then
			buf_wa <= (others => '0');
		elsif(din_we = '1' or en_din = '0')then
			buf_wa(1 downto 0) <= buf_wa(1 downto 0) + 1;
			if(buf_wa(1 downto 0) = "11" or din(64) = '1')then
				buf_wa(12 downto 2) <= buf_wa(12 downto 2) + 1;
			end if;
		end if;
		if(reset = '1' or en_din = '0')then
			got_eoe <= '0';
		elsif(din(64) = '1' and din_we = '1')then
			got_eoe <= '1';
		end if;
		buf_ra2SyncRegs <= buf_ra2SyncRegs(2 downto 0) & buf_ra(2);
		if(reset = '0')then
			buf_rap <= (others => '0');
		elsif(buf_ra2SyncRegs(3) /= buf_ra2SyncRegs(2))then
			buf_rap <= buf_rap + 1;
		end if;
		buf_full <= and_reduce(buf_wa(12 downto 4) - buf_rap);
	end if;
end process;
buf_di <= "0000000" & din;
g_wbuf : for i in 0 to 3 generate
	i_wbuf : BRAM_SDP_MACRO
   generic map (
      BRAM_SIZE => "36Kb", -- Target BRAM, "18Kb" or "36Kb" 
      DEVICE => "7SERIES", -- Target device: "VIRTEX5", "VIRTEX6", "7SERIES", "SPARTAN6" 
      WRITE_WIDTH => 72,    -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      READ_WIDTH => 72,     -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      DO_REG => 1, -- Optional output register (0 or 1)
      SIM_COLLISION_CHECK => "NONE", -- Collision check enable "ALL", "WARNING_ONLY", 
                                    -- "GENERATE_X_ONLY" or "NONE"       
      WRITE_MODE => "WRITE_FIRST", -- Specify "READ_FIRST" for same clock or synchronous clocks
                                   --  Specify "WRITE_FIRST for asynchrononous clocks on ports
      INIT => X"000000000000000000") --  Initial values on output port
   port map (
      DO => buf_do(i*72+71 downto i*72),         -- Output read data port, width defined by READ_WIDTH parameter
      DI => buf_di,         -- Input write data port, width defined by WRITE_WIDTH parameter
      RDADDR => buf_ra(8 downto 0), -- Input read address, width defined by read port depth
      RDCLK => memclk,   -- 1-bit input read clock
      RDEN => bufRdEn,     -- 1-bit input read port enable
      REGCE => bufREGCE,   -- 1-bit input read output register enable
      RST => '0',       -- 1-bit input reset 
      WE => x"ff",         -- Input write enable, width defined by write port depth
      WRADDR => buf_wa(10 downto 2), -- Input write address, width defined by write port depth
      WRCLK => sysclk,   -- 1-bit input write clock
      WREN => buf_we(i)      -- 1-bit input write port enable
   );
end generate;
buf_we(0) <= din_we and not buf_wa(1) and not buf_wa(0);
buf_we(1) <= din_we and not buf_wa(1) and buf_wa(0);
buf_we(2) <= din_we and buf_wa(1) and not buf_wa(0);
buf_we(3) <= din_we and ((buf_wa(1) and buf_wa(0)) or din(64));
bufRdEn <= not buf_do_vld or app_wdf_accept;
bufREGCE <= not dout_vld or app_wdf_accept;
process(memclk)
begin
	if(memclk'event and memclk = '1')then
		if(resetMem = '1')then
			buf_wa2SyncRegs <= (others => '0');
			got_eoeSyncRegs <= (others => '0');
			buf_wap <= (others => '0');
			buf_ra <= (others => '0');
			EventDataDone <= '0';
			EventDone <= '0';
			rqst_i <= '0';
			buf_do_vld <= '0';
			dout_vld <= '0';
			dout_wc <= (others => '0');
			en_app_en <= '0';
		else
			if(EventDone = '1')then
				buf_wa2SyncRegs <= (others => '0');
				got_eoeSyncRegs <= (others => '0');
			else
				buf_wa2SyncRegs <= buf_wa2SyncRegs(2 downto 0) & buf_wa(2);
				got_eoeSyncRegs <= got_eoeSyncRegs(3 downto 0) & got_eoe;
			end if;
			if(EventDone = '1')then
				buf_wap <= (others => '0');
			elsif(buf_wa2SyncRegs(3) /= buf_wa2SyncRegs(2))then
				buf_wap <= buf_wap + 1;
			end if;
			if(EventDone = '1')then
				buf_ra <= (others => '0');
			elsif(buf_re = '1')then
				buf_ra <= buf_ra + 1;
			end if;
			if(buf_do(280) = '1' and app_wdf_accept = '1')then
				EventDataDone <= '1';
			elsif(en_app_en = '0')then
				EventDataDone <= '0';
			end if;
			EventDone <= EventDataDone and not en_app_en;
			if(run = '0' or EventDataDone = '1' or EventDone = '1' or ((buf_do(280) = '1' or and_reduce(dout_wc) = '1') and app_wdf_accept = '1'))then
				rqst_i <= '0';
			elsif((got_eoeSyncRegs(4) = '1' or buf_wap(10 downto 5) > buf_ra(10 downto 5)) and en_app_en = '0')then
				rqst_i <= '1';
			end if;
			if(EventDone = '1')then
				buf_do_vld <= '0';
			elsif(buf_wap > buf_ra)then
				buf_do_vld <= '1';
			elsif(dout_vld = '0')then
				buf_do_vld <= '0';
			end if;
			if(EventDone = '1')then
				dout_vld <= '0';
			elsif(buf_do_vld = '1')then
				dout_vld <= '1';
			elsif(app_wdf_accept = '1')then
				dout_vld <= '0';
			end if;
			if(EventDone = '1')then
				dout_wc <= (others => '0');
			elsif(app_wdf_accept = '1')then
				dout_wc <= dout_wc + 1;
			end if;
			if(rqst_i = '1' and ack = '0')then
				if(buf_wap(10 downto 5) > buf_ra(10 downto 5))then
					wc_left <= "100000";
				else
					wc_left <= '0' & buf_wap(4 downto 0);
				end if;
			elsif(app_rdy = '1' and en_app_en = '1')then
				wc_left <= wc_left - 1;
			end if;
			if(rqst_i = '1' and ack = '0')then
				en_app_en <= '1';
			elsif(or_reduce(wc_left(5 downto 1)) = '0' and (app_rdy = '1' or wc_left(0) = '0'))then
				en_app_en <= '0';
			end if;
		end if;
	end if;
end process;
process(memclk)
begin
	if(memclk'event and memclk = '1')then
			if(ack = '0')then
				app_addr_i(4 downto 0) <= (others => '0');
				app_addr_i(10 downto 5) <= buf_ra(10 downto 5);
			elsif(app_rdy = '1')then
				app_addr_i(4 downto 0) <= app_addr_i(4 downto 0) + 1;
			end if;
	end if;
end process;
app_wdf_accept <= app_wdf_rdy and ack and rqst_i;
dout <= buf_do(279 downto 216) & buf_do(207 downto 144) & buf_do(135 downto 72) & buf_do(63 downto 0);
buf_re <= '1' when (app_wdf_accept = '1' or buf_do_vld = '0') and buf_wap > buf_ra else '0';
end Behavioral;

