----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:02:29 01/25/2013 
-- Design Name: 
-- Module Name:    ddr_wportA - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.std_logic_misc.all;
use work.amc13_pack.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;
Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity ddr_wportB is
    Port ( memclk : in  STD_LOGIC;
           sysclk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           resetSys : in  STD_LOGIC;
           resetMem : in  STD_LOGIC;
           test : in  STD_LOGIC_VECTOR(1 downto 0); -- memory test
					 test_block_sent	: out  STD_LOGIC := '0'; -- indicating test writing progress. One pulse every 256 32-bit words
					 test_pause : in std_logic;
-- each TCP segment sent will have dest IP address (32-bit), TCP port(2x16-bit), SEQ number(32-bit), length(16 bit), time stamp(16 bit), control bits(6)
-- and memory starting address of the segment payload data(24-bit)
-- for alignment purposes, each entry occupies 32 bytes of memory space (256 bits)
           TCP_din : in  array3X32; -- TCP data are written in unit of 32-bit words
           TCP_addr : in  array3X13; -- each XGbE channel can store 8192 entries
           TCP_rqst : in  STD_LOGIC_VECTOR (2 downto 0);
           TCP_re : out  STD_LOGIC_VECTOR (2 downto 0);
--	ipbus signals
					 ipb_clk : in  STD_LOGIC;
					 ipb_write : in  STD_LOGIC;
					 ipb_strobe	: in  STD_LOGIC;
					 ipb_addr	: in  STD_LOGIC_VECTOR(31 downto 0);
					 ipb_wdata : in  STD_LOGIC_VECTOR(31 downto 0);
					 ipb_ack : out  STD_LOGIC;
					 run : in  STD_LOGIC; -- ipbus write is allowed only when run = '0'
-- ddr3 user interface
           app_rqst : out  STD_LOGIC; -- request to output data
           app_ack : in  STD_LOGIC; -- permission to output data
           app_en_accept : in  STD_LOGIC;
           app_en : out  STD_LOGIC;
           app_wdf_rdy : in  STD_LOGIC;
           app_wdf_wren : out  STD_LOGIC;
					 app_wdf_mask : out  STD_LOGIC_VECTOR(7 downto 0);
           app_addr : out  STD_LOGIC_VECTOR (23 downto 0) := (others => '0'); -- starting address of write burst(1Kbytes or until end of event)
           dout : out  STD_LOGIC_VECTOR (255 downto 0) := (others => '0');
           debug_out : out  STD_LOGIC_VECTOR (63 downto 0)
					 );
end ddr_wportB;

architecture Behavioral of ddr_wportB is
signal TCP_re_i : std_logic_vector(2 downto 0) := (others => '0');
--type ram32x47 is array (0 to 31) of std_logic_vector(46 downto 0);
--signal TCPqueue : ram32x47 := (others => (others => '0'));
signal TCPqueue_di : std_logic_vector(46 downto 0) := (others => '0');
signal TCPqueue_dop : std_logic_vector(46 downto 0) := (others => '0');
signal TCPqueue_do : std_logic_vector(46 downto 0) := (others => '0');
signal TCPqueue_wa : std_logic_vector(4 downto 0) := (others => '0');
signal TCPqueue_ra : std_logic_vector(4 downto 0) := (others => '0');
signal TCP_wc : std_logic_vector(2 downto 0) := (others => '0');
signal TCPqueue_we : std_logic := '0';
signal TCPqueue_full : std_logic := '0';
signal TCPqueue_wap : std_logic_vector(4 downto 0) := (others => '0');
signal TCPqueue_wa0SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal TCPqueue_wa1SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal TCPqueue_rap : std_logic_vector(4 downto 0) := (others => '0');
signal TCPqueue_ra0SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal TCPqueue_ra1SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal ipb_weToggle : std_logic := '0';
signal ipb_di : std_logic_vector(31 downto 0) := (others => '0');
signal ipb_addr_q : std_logic_vector(26 downto 0) := (others => '0');
signal ipb_weToggleSyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal testSyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal lfsr: std_logic_vector(31 downto 0) := (others => '0');
--signal addr_vld : std_logic := '0';
--signal dout_vld : std_logic := '0';
signal app_rqst_i : std_logic := '0';
signal data_mux : std_logic_vector(31 downto 0) := (others => '0');
signal data_we : std_logic_vector(7 downto 0) := (others => '0');
type array8X4 is array (0 to 7) of std_logic_vector(3 downto 0);
signal data_a : array8X4 := (others => (others => '1'));
signal data_in : std_logic_vector(263 downto 0) := (others => '0');
signal data_out : std_logic_vector(263 downto 0) := (others => '0');
signal addr_we : std_logic := '0';
signal addr_full : std_logic := '0';
signal addr_a : std_logic_vector(3 downto 0) := (others => '1');
signal addr_in : std_logic_vector(23 downto 0) := (others => '0');
signal addr_out : std_logic_vector(23 downto 0) := (others => '0');
signal app_addr0SyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal test_pauseSyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal app_addr_i : std_logic_vector(23 downto 0) := (others => '0');
signal wc : std_logic_vector(2 downto 0) := (others => '0');
signal app_wdf_accept : std_logic := '0';
--signal app_en_accept : std_logic := '0';
signal en_app_en : std_logic := '0';
signal en_app_en_q : std_logic := '0';
signal app_addr0 : std_logic := '0';

begin
debug_out(0) <= TCPqueue_full;
debug_out(1) <= testSyncRegs(2);
debug_out(9 downto 2) <= lfsr(7 downto 0);
debug_out(14 downto 10) <= TCPqueue_rap;
debug_out(19 downto 15) <= TCPqueue_wa;
debug_out(24 downto 20) <= TCPqueue_ra;
debug_out(29 downto 25) <= TCPqueue_wap;
debug_out(30) <= addr_full;
debug_out(33 downto 31) <= wc;
debug_out(37 downto 34) <= addr_a;
debug_out(41 downto 38) <= data_a(0);
debug_out(45 downto 42) <= data_a(1);
debug_out(49 downto 46) <= data_a(2);
debug_out(53 downto 50) <= data_a(4);
debug_out(57 downto 54) <= data_a(6);
debug_out(61 downto 58) <= data_a(7);
debug_out(62) <= app_rqst_i;
debug_out(63) <= en_app_en;
app_rqst <= app_rqst_i or en_app_en;
app_addr <= app_addr_i;
app_wdf_wren <= app_ack and app_rqst_i;
app_en <= app_ack and en_app_en;
TCP_re <= TCP_re_i;
process(ipb_clk)
begin
	if(ipb_clk'event and ipb_clk = '1')then
		if(reset = '1')then
			ipb_weToggle <= '0';
		elsif(ipb_strobe = '1' and ipb_write = '1' and ipb_addr(27) = '1' and run = '0')then
			ipb_weToggle <= not ipb_weToggle;
		end if;
		ipb_di <= ipb_wdata;
		ipb_addr_q <= ipb_addr(26 downto 0);
		if(ipb_strobe = '1' and ipb_write = '1' and ipb_addr(27) = '1')then
			ipb_ack <= '1';
		else
			ipb_ack <= '0';
		end if;
	end if;
end process;
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		testSyncRegs <= testSyncRegs(1 downto 0) & test(0);
--		test_block_sent <= not resetSys and (app_addr0SyncRegs(3) and not app_addr0SyncRegs(2));
		test_block_sent <= not resetSys and (app_addr0SyncRegs(3) xor app_addr0SyncRegs(2));
		if(resetSys = '1')then
			app_addr0SyncRegs <= (others => '0');
		else
--			app_addr0SyncRegs <= app_addr0SyncRegs(2 downto 0) & app_rqst_i;
			app_addr0SyncRegs <= app_addr0SyncRegs(2 downto 0) & app_addr0;
		end if;
		if(resetSys = '1')then
			TCPqueue_full <= '0';
		elsif((TCPqueue_wa(4 downto 1) - TCPqueue_rap(4 downto 1)) > x"9")then
			TCPqueue_full <= '1';
		else
			TCPqueue_full <= '0';
		end if;
		if(resetSys = '1' or run = '0')then
			TCP_wc <= "000";
		elsif(TCP_re_i /= "000")then
			TCP_wc <= TCP_wc + 1;
		end if;
		if(resetSys = '1' or run = '0')then
			TCP_re_i <= "000";
		elsif(TCP_re_i(0) = '1' and TCP_wc = "111")then
			TCP_re_i(0) <= '0';
			if(TCP_rqst(1) = '1')then
				TCP_re_i(1) <= '1';
			elsif(TCP_rqst(2) = '1')then
				TCP_re_i(2) <= '1';
			end if;
		elsif(TCP_re_i(1) = '1' and TCP_wc = "111")then
			TCP_re_i(1) <= '0';
			if(TCP_rqst(2) = '1')then
				TCP_re_i(2) <= '1';
			elsif(TCP_rqst(0) = '1')then
				TCP_re_i(0) <= '1';
			end if;
		elsif(TCP_re_i(2) = '1' and TCP_wc = "111")then
			TCP_re_i(2) <= '0';
			if(TCP_rqst(0) = '1')then
				TCP_re_i(0) <= '1';
			elsif(TCP_rqst(1) = '1')then
				TCP_re_i(1) <= '1';
			end if;
		elsif(TCPqueue_full = '0' and TCP_re_i = "000")then
			if(TCP_rqst(0) = '1')then
				TCP_re_i(0) <= '1';
			elsif(TCP_rqst(1) = '1')then
				TCP_re_i(1) <= '1';
			elsif(TCP_rqst(2) = '1')then
				TCP_re_i(2) <= '1';
			end if;
		end if;
		if(run = '0')then
			TCPqueue_di <= "11" & lfsr & "0000000000000";
		elsif(TCP_re_i(2) = '1')then
			TCPqueue_di <= "10" & TCP_din(2) & TCP_addr(2);
		elsif(TCP_re_i(1) = '1')then
			TCPqueue_di <= "01" & TCP_din(1) & TCP_addr(1);
		else
			TCPqueue_di <= "00" & TCP_din(0) & TCP_addr(0);
		end if;
		if(resetSys = '1')then
			TCPqueue_we <= '0';
		elsif(TCP_re_i /= "000" or (testSyncRegs(2) = '1' and run = '0' and TCPqueue_full = '0'))then
			TCPqueue_we <= '1';
		else
			TCPqueue_we <= '0';
		end if;
--		if(TCPqueue_we = '1')then
--			TCPqueue(conv_integer(TCPqueue_wa)) <= TCPqueue_di;
--		end if;
		if(resetSys = '1')then
			TCPqueue_wa <= (others => '0');
		elsif(TCPqueue_we = '1')then
			TCPqueue_wa(1) <= TCPqueue_wa(0);
			TCPqueue_wa(0) <= not TCPqueue_wa(1);
			if(TCPqueue_wa(1 downto 0) = "10")then
				TCPqueue_wa(4 downto 2) <= TCPqueue_wa(4 downto 2) + 1;
			end if;
		end if;
		if(resetSys = '1')then
			TCPqueue_ra0SyncRegs <= (others => '0');
			TCPqueue_ra1SyncRegs <= (others => '0');
			TCPqueue_rap <= (others => '0');
		else
			TCPqueue_ra0SyncRegs <= TCPqueue_ra0SyncRegs(1 downto 0) & TCPqueue_ra(0);
			TCPqueue_ra1SyncRegs <= TCPqueue_ra1SyncRegs(1 downto 0) & TCPqueue_ra(1);
			TCPqueue_rap(1) <= TCPqueue_ra1SyncRegs(2);
			TCPqueue_rap(0) <= TCPqueue_ra0SyncRegs(2);
			if(TCPqueue_rap(1) = '1' and TCPqueue_ra1SyncRegs(2) = '0')then
				TCPqueue_rap(4 downto 2) <= TCPqueue_rap(4 downto 2) + 1;
			end if;
		end if;
		if(testSyncRegs(2) = '0')then
			lfsr <= (others => '0');
		elsif(TCPqueue_full = '0')then
			if(test(1) = '0')then
				lfsr <= lfsr(30 downto 0) & not(lfsr(31) xor lfsr(21) xor lfsr(1) xor lfsr(0));
			else
				lfsr <= lfsr + 1;
			end if;
		end if;
	end if;
end process;
g_TCPqueue : for i in 0 to 46 generate
   i_TCPqueue : RAM32X1D
   port map (
      DPO => TCPqueue_dop(i),     -- Read-only 1-bit data output
      SPO => open,     -- R/W 1-bit data output
      A0 => TCPqueue_wa(0),       -- R/W address[0] input bit
      A1 => TCPqueue_wa(1),       -- R/W address[1] input bit
      A2 => TCPqueue_wa(2),       -- R/W address[2] input bit
      A3 => TCPqueue_wa(3),       -- R/W address[3] input bit
      A4 => TCPqueue_wa(4),       -- R/W address[4] input bit
      D => TCPqueue_di(i),         -- Write 1-bit data input
      DPRA0 => TCPqueue_ra(0), -- Read-only address[0] input bit
      DPRA1 => TCPqueue_ra(1), -- Read-only address[1] input bit
      DPRA2 => TCPqueue_ra(2), -- Read-only address[2] input bit
      DPRA3 => TCPqueue_ra(3), -- Read-only address[3] input bit
      DPRA4 => TCPqueue_ra(4), -- Read-only address[4] input bit
      WCLK => sysclk,   -- Write clock input
      WE => TCPqueue_we       -- Write enable input
   );
end generate;
process(memclk)
begin
	if(memclk'event and memclk = '1')then
		if(TCPqueue_wap /= TCPqueue_ra)then
--			TCPqueue_do <= TCPqueue(conv_integer(TCPqueue_ra));
			TCPqueue_do <= TCPqueue_dop;
		end if;
		if(resetMem = '1')then
			TCPqueue_ra <= (others => '0');
		elsif(TCPqueue_wap /= TCPqueue_ra and addr_full = '0')then
			TCPqueue_ra(1) <= TCPqueue_ra(0);
			TCPqueue_ra(0) <= not TCPqueue_ra(1);
			if(TCPqueue_ra(1 downto 0) = "10")then
				TCPqueue_ra(4 downto 2) <= TCPqueue_ra(4 downto 2) + 1;
			end if;
		end if;
		if(resetMem = '1')then
			TCPqueue_wa0SyncRegs <= (others => '0');
			TCPqueue_wa1SyncRegs <= (others => '0');
			TCPqueue_wap <= (others => '0');
		else
			TCPqueue_wa0SyncRegs <= TCPqueue_wa0SyncRegs(1 downto 0) & TCPqueue_wa(0);
			TCPqueue_wa1SyncRegs <= TCPqueue_wa1SyncRegs(1 downto 0) & TCPqueue_wa(1);
			TCPqueue_wap(1) <= TCPqueue_wa1SyncRegs(2);
			TCPqueue_wap(0) <= TCPqueue_wa0SyncRegs(2);
			if(TCPqueue_wap(1) = '1' and TCPqueue_wa1SyncRegs(2) = '0')then
				TCPqueue_wap(4 downto 2) <= TCPqueue_wap(4 downto 2) + 1;
			end if;
		end if;
		if(resetMem = '1')then
			ipb_weToggleSyncRegs <= (others => '0');
			test_pauseSyncRegs <= (others => '0');
		else
			ipb_weToggleSyncRegs <= ipb_weToggleSyncRegs(2 downto 0) & ipb_weToggle;
			test_pauseSyncRegs <= test_pauseSyncRegs(1 downto 0) & test_pause;
		end if;
		if(resetMem = '1')then
			addr_we <= '0';
		elsif(ipb_weToggleSyncRegs(3) /= ipb_weToggleSyncRegs(2) or (TCPqueue_wap /= TCPqueue_ra and wc = "111"))then
			addr_we <= '1';
		else
			addr_we <= '0';
		end if;
		if(resetMem = '1')then
			addr_full <= '0';
		elsif(wc = "111" and addr_a = x"d" and TCPqueue_wap /= TCPqueue_ra)then
			addr_full <= '1';
		elsif(addr_a = x"c")then
			addr_full <= '0';
		end if;
		if(resetMem = '1')then
			wc <= "000";
		elsif(TCPqueue_wap /= TCPqueue_ra and addr_full = '0')then
			wc <= wc + 1;
		end if;
		if(resetMem = '1')then
			addr_in <= (others => '0');
		elsif(run = '0')then
			if(test(0) = '0' and ipb_weToggleSyncRegs(3) /= ipb_weToggleSyncRegs(2))then
				addr_in <= ipb_addr_q(26 downto 3);
			elsif(addr_we = '1')then
				addr_in <= addr_in + 1;
			end if;
		else
			addr_in <= TCPqueue_do(46 downto 45) & "111111111" & TCPqueue_do(12 downto 0);
		end if;
		if(resetMem = '1')then
			data_we <= (others => '0');
		elsif(ipb_weToggleSyncRegs(3) /= ipb_weToggleSyncRegs(2))then
			data_we <= (others => '1');
		else
			data_we <= (others => '0');
			if(TCPqueue_wap /= TCPqueue_ra and addr_full = '0')then
				data_we(conv_integer(wc)) <= '1';
			end if;
		end if;
		for i in 0 to 7 loop
			if(resetMem = '1')then
				data_a(i) <= (others => '1');
			elsif(data_we(i) = '1' and (and_reduce(data_a(i)) = '1' or app_wdf_accept = '0'))then
				data_a(i) <= data_a(i) + 1;
			elsif(data_we(i) = '0' and and_reduce(data_a(i)) = '0' and app_wdf_accept = '1')then
				data_a(i) <= data_a(i) - 1;
			end if;
		end loop;
--		if(resetSys = '1')then
--			dout_vld <= '0';
--		elsif(and_reduce(data_a) = '0')then
--			dout_vld <= '1';
--		elsif(app_wdf_accept = '1')then
--			dout_vld <= '0';
--		end if;
--		if(and_reduce(data_a) = '0' and (dout_vld = '0' or app_wdf_accept = '1'))then
		for i in 0 to 7 loop
			dout(32*i+31 downto 32*i) <= data_out(33*i+31 downto 33*i);
			app_wdf_mask(i) <= data_out(33*i+32);
		end loop;
--		end if;
		if(resetMem = '1')then
			addr_a <= (others => '1');
--		elsif(addr_we = '1' and (and_reduce(addr_a) = '1' or (addr_vld = '1' and app_en_accept = '0')))then
--			addr_a <= addr_a + 1;
--		elsif(addr_we = '0' and and_reduce(addr_a) = '0' and (addr_vld = '0' or app_en_accept = '1'))then
		elsif(addr_we = '1' and (and_reduce(addr_a) = '1' or app_en_accept = '0'))then
			addr_a <= addr_a + 1;
		elsif(addr_we = '0' and and_reduce(addr_a) = '0' and app_en_accept = '1')then
			addr_a <= addr_a - 1;
		end if;
--		if(resetSys = '1')then
--			addr_vld <= '0';
--		elsif(and_reduce(addr_a) = '0')then
--			addr_vld <= '1';
--		elsif(app_en_accept = '1')then
--			addr_vld <= '0';
--		end if;
--		if(and_reduce(addr_a) = '0' and (addr_vld = '0' or app_en_accept = '1'))then
		app_addr_i <= addr_out;
--		end if;
		if(resetMem = '1')then
			en_app_en <= '0';
		elsif(app_rqst_i = '1' and app_ack = '0')then
			en_app_en <= '1';
		elsif(app_en_accept = '1')then
			en_app_en <= '0';
		end if;
		en_app_en_q <= en_app_en;
		if(resetMem = '1')then
			app_addr0 <= '0';
		elsif(en_app_en_q = '1' and en_app_en = '0')then
			app_addr0 <= not app_addr0;
		end if;
		if(resetMem = '1')then
			app_rqst_i <= '0';
		elsif(and_reduce(data_a(7)) = '0' and and_reduce(addr_a) = '0' and app_rqst_i = '0' and en_app_en = '0' and test_pauseSyncRegs(2) = '0')then
			app_rqst_i <= '1';
		elsif(app_wdf_rdy = '1' and app_ack = '1')then
			app_rqst_i <= '0';
		end if;
	end if;
end process;
g_data_buf: for i in 0 to 263 generate
	i_data_buf : SRL16E
   port map (
      Q => data_out(i),       -- SRL data output
      A0 => data_a(i/33)(0),     -- Select[0] input
      A1 => data_a(i/33)(1),     -- Select[1] input
      A2 => data_a(i/33)(2),     -- Select[2] input
      A3 => data_a(i/33)(3),     -- Select[3] input
      CE => data_we(i/33),     -- Clock enable input
      CLK => memclk,   -- Clock input
      D => data_in(i)        -- SRL data input
   );
end generate;
g_addr_buf: for i in 0 to 23 generate
	i_addr_buf : SRL16E
   port map (
      Q => addr_out(i),       -- SRL data output
      A0 => addr_a(0),     -- Select[0] input
      A1 => addr_a(1),     -- Select[1] input
      A2 => addr_a(2),     -- Select[2] input
      A3 => addr_a(3),     -- Select[3] input
      CE => addr_we,     -- Clock enable input
      CLK => memclk,   -- Clock input
      D => addr_in(i)        -- SRL data input
   );
end generate;
data_mux <= ipb_di when run = '0' and test(0) = '0' else TCPqueue_do(44 downto 13);
process(data_mux,ipb_addr,run,test)
begin
	for i in 0 to 7 loop
		data_in(i*33+31 downto i*33) <= data_mux;
		if(conv_integer(ipb_addr_q(2 downto 0)) /= i and run = '0' and test(0) = '0')then
			data_in(i*33+32) <= '1';
		else
			data_in(i*33+32) <= '0';
		end if;
	end loop;
end process;
--app_en_accept <= app_rdy and app_ack and en_app_en;
app_wdf_accept <= app_wdf_rdy and app_ack and app_rqst_i;
end Behavioral;

