----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:31:57 12/05/2012 
-- Design Name: 
-- Module Name:    link_status - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.std_logic_misc.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity link_status is
    Port ( clk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           send_LBLOCK : in  STD_LOGIC;
           E_R : in  STD_LOGIC_VECTOR (7 downto 0);
           rxdvld : in  STD_LOGIC;
           rxsync : in  STD_LOGIC_VECTOR (1 downto 0);
           local_fault : in  STD_LOGIC_VECTOR (1 downto 0);
           remote_fault : in  STD_LOGIC_VECTOR (1 downto 0);
           link_fault : out  STD_LOGIC_VECTOR (1 downto 0));
end link_status;

architecture Behavioral of link_status is
type type_state is (INIT, COUNT, FAULT, NEW_FAULT_TYPE);
signal state : type_state := INIT;
signal local_fault_r : std_logic_vector(1 downto 0) := (others => '0');
signal remote_fault_r : std_logic_vector(1 downto 0) := (others => '0');
signal col_cnt : std_logic_vector(7 downto 0) := (others => '0');
signal seq_cnt : std_logic_vector(1 downto 0) := (others => '0');
signal last_seq_type : std_logic_vector(1 downto 0) := (others => '0');

begin
process(clk)
begin
	if(clk'event and clk = '1')then
		if(rxsync /= "00")then
			if(send_LBLOCK = '1')then
				local_fault_r <= "11";
			elsif(or_reduce(E_R) = '1')then
				local_fault_r <= "00";
			else
				local_fault_r <= local_fault;
			end if;
			if(or_reduce(E_R) = '1')then
				remote_fault_r <= "00";
			else
				remote_fault_r <= remote_fault;
			end if;
		elsif(rxdvld = '1')then
			local_fault_r <= local_fault_r(0) & '0';
			remote_fault_r <= remote_fault_r(0) & '0';
		end if;
		if(rxdvld = '1')then
			if(local_fault_r(1) = '1')then
				last_seq_type <= "01";
			elsif(remote_fault_r(1) = '1')then
				last_seq_type <= "10";
			end if;
		end if;
		if(rxdvld = '1')then
			if(local_fault_r(1) = '1' or remote_fault_r(1) = '1')then
				col_cnt <= (others => '0');
			else
				col_cnt <= col_cnt + 1;
			end if;
			if(state = INIT or state = NEW_FAULT_TYPE)then
				seq_cnt <= (others => '0');
			elsif(local_fault_r(1) = '1' or remote_fault_r(1) = '1')then
				seq_cnt <= seq_cnt + 1;
			end if;
		end if;
		if(reset = '1')then
			state <= INIT;
		elsif(rxdvld = '1')then
			case state is
				when INIT =>
					link_fault <= "00";
					if(local_fault_r(1) = '1' or remote_fault_r(1) = '1')then
						state <= COUNT;
					end if;
				when COUNT =>
					if(local_fault_r(1) = '0' and remote_fault_r(1) = '0' and col_cnt(7) = '1')then
						state <= INIT;
					end if;
					if(local_fault_r(1) = '1')then
						if(last_seq_type(1) = '1')then
							state <= NEW_FAULT_TYPE;
						elsif(seq_cnt = "11")then
							state <= FAULT;
						end if;
					elsif(remote_fault_r(1) = '1')then
						if(last_seq_type(0) = '1')then
							state <= NEW_FAULT_TYPE;
						elsif(seq_cnt = "11")then
							state <= FAULT;
						end if;
					end if;
				when FAULT =>
					link_fault <= last_seq_type;
					if(local_fault_r(1) = '0' and remote_fault_r(1) = '0' and col_cnt(7) = '1')then
						state <= INIT;
					end if;
				when others =>
					state <= COUNT;
			end case;
		end if;
	end if;
end process;

end Behavioral;

