`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    08:57:35 01/07/2010 
// Design Name: 
// Module Name:    gbe_packetbuffer 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module gbe_rxpacketbuffer(
    input [7:0] mac_rxd,
    input mac_rxdv, reset,
    input mac_rxpacketok,
    input mac_rxpacketbad,
    input mac_clk, clk,
    output [7:0] packet_rxd,
    input [10:0] packet_rxa,
    output reg [10:0] packet_len,
    output reg packet_rxready,
    input packet_rxdone
    );

	reg [10:0] packet_len_acc;
	reg [10:0] packet_len_fifo[7:0]; // store up to eight incoming packets at once
	reg [2:0] packet_len_wp, packet_len_rp;
	wire packet_len_fifo_empty;

  assign packet_len_fifo_empty=(packet_len_wp==packet_len_rp); // clock domain crossing - sync reg below

	reg [7:0] inByteBuffer;
	reg was_packetok, was_rxdone, was_dv;

	reg [11:0] buffer_write_base, buffer_read_base;
	reg [11:0] buffer_addr_write;

//assign buffer_addr_write = buffer_write_base+packet_len_acc;

always @(posedge mac_clk) buffer_addr_write<=buffer_write_base+packet_len_acc;
always @(posedge mac_clk) inByteBuffer<=mac_rxd;
always @(posedge mac_clk) was_dv<=mac_rxdv;
always @(posedge mac_clk) was_packetok<=mac_rxpacketok;

always @(posedge mac_clk) 
	if (reset) begin
		buffer_write_base<=0;
		packet_len_wp<=0;
		packet_len_acc<=0;
	end else if (mac_rxpacketok && !was_packetok) begin // keep the packet
		packet_len_fifo[packet_len_wp]<=packet_len_acc;
		buffer_write_base<=buffer_write_base+packet_len_acc;
	end else if (!mac_rxpacketok && was_packetok) begin // keep the packet (inc count)
		packet_len_wp<=packet_len_wp+1;
		packet_len_acc<=0;
	end else if (mac_rxpacketbad) begin // drop the packet (forget the length...)
		packet_len_acc<=0;
	end else if (mac_rxdv) begin
		packet_len_acc<=packet_len_acc+1;		
	end 

	wire [11:0] buffer_addr_read;
	
assign buffer_addr_read=buffer_read_base+packet_rxa;

always @(posedge clk) begin
	packet_len<=packet_len_fifo[packet_len_rp];
	packet_rxready<=!packet_len_fifo_empty;
	was_rxdone<=packet_rxdone;
end
always @(posedge clk) 
	if (reset) begin
		packet_len_rp<=0;
		buffer_read_base<=0;
	end else if (packet_rxdone && !was_rxdone) begin 
		packet_len_rp<=packet_len_rp+1;
		buffer_read_base<=buffer_read_base+packet_len;
	end
	
//reg [7:0] RAM [0:4095];
//reg [7:0] fifoOut;
//assign packet_rxd = fifoOut;
//always @(posedge mac_clk) begin
//	if(mac_rxdv || was_dv) RAM[buffer_addr_write] <= inByteBuffer;
//end
//always @(posedge clk) begin
//	fifoOut <= RAM[buffer_addr_read];
//end
   BRAM_SDP_MACRO #(
      .BRAM_SIZE("36Kb"), // Target BRAM, "18Kb" or "36Kb" 
      .DEVICE("7SERIES"), // Target device: "VIRTEX5", "VIRTEX6", "SPARTAN6", "7SERIES" 
      .WRITE_WIDTH(8),    // Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      .READ_WIDTH(8),     // Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      .DO_REG(0),         // Optional output register (0 or 1)
      .INIT_FILE ("NONE"),
      .SIM_COLLISION_CHECK ("ALL"), // Collision check enable "ALL", "WARNING_ONLY", 
                                    //   "GENERATE_X_ONLY" or "NONE" 
      .SRVAL(72'h000000000000000000), // Set/Reset value for port output
      .INIT(72'h000000000000000000),  // Initial values on output port
      .WRITE_MODE("WRITE_FIRST")  // Specify "READ_FIRST" for same clock or synchronous clocks
                                   //   Specify "WRITE_FIRST for asynchronous clocks on ports
   ) fifo (
      .DO(packet_rxd),         // Output read data port, width defined by READ_WIDTH parameter
      .DI(inByteBuffer),         // Input write data port, width defined by WRITE_WIDTH parameter
      .RDADDR(buffer_addr_read), // Input read address, width defined by read port depth
      .RDCLK(clk),   // 1-bit input read clock
      .RDEN(1'b1),     // 1-bit input read port enable
      .REGCE(1'b1),   // 1-bit input read output register enable
      .RST(1'b0),       // 1-bit input reset      
      .WE(1'b1),         // Input write enable, width defined by write port depth
      .WRADDR(buffer_addr_write), // Input write address, width defined by write port depth
      .WRCLK(mac_clk),   // 1-bit input write clock
      .WREN(mac_rxdv || was_dv)      // 1-bit input write port enable
   );
endmodule
