-- Dave Newbold, May 2011
--
-- $Id$

library ieee;
use ieee.std_logic_1164.all;
use work.ipbus.all;
use work.ipbus_ctrl_decl.all;
use work.bus_arb_decl.all;

entity ipbus_ctrl is 
  generic(NOOB: positive := 1);
  port(
	 ipb_clk: in std_logic;
	 rst_ipb: in std_logic;
	 rst_macclk: in std_logic;
	 mac_txclk: in std_logic;
	 mac_rxclk: in std_logic;
	 mac_rxd: in std_logic_vector(7 downto 0);
	 mac_rxdvld: in std_logic;
	 mac_rxgoodframe: in std_logic;
	 mac_rxbadframe: in std_logic;
	 mac_txd: out std_logic_vector(7 downto 0);
	 mac_txdvld: out std_logic;
	 mac_txack: in std_logic;
	 ipb_out: out ipb_wbus;
	 ipb_in: in ipb_rbus;
	 mac_addr: in std_logic_vector(47 downto 0);
	 ip_addr: in std_logic_vector(31 downto 0));

end ipbus_ctrl;

architecture rtl of ipbus_ctrl is

  signal packet_txd, packet_rxd: std_logic_vector(7 downto 0);
  signal packet_txa, packet_txlen, packet_rxa, packet_rxl: std_logic_vector(10 downto 0);
  signal packet_txwe, packet_txdone, packet_rxready, packet_rxdone: std_logic;
  signal arp_rxa, arp_txa: std_logic_vector(5 downto 0);
  signal arp_txd: std_logic_vector(7 downto 0);
  signal arp_we, arp_xmit, arp_done, arp_ready: std_logic;

  signal icmp_rxa, icmp_txa, icmp_len: std_logic_vector(9 downto 0);
  signal icmp_txd: std_logic_vector(7 downto 0);
  signal icmp_we, icmp_xmit, icmp_done, icmp_ready: std_logic;

  signal udp_rxa, udp_txa, udp_len: std_logic_vector(10 downto 0);
  signal udp_txd: std_logic_vector(7 downto 0);
  signal udp_we, udp_xmit, udp_done, udp_ready, udp_space, udp_xmit_req, udp_xmit_ok: std_logic;

  signal packet_req_addr,packet_req_len,packet_resp_addr,packet_resp_len: std_logic_vector(8 downto 0);
  signal packet_req_data,packet_resp_data: std_logic_vector(31 downto 0);
  signal resp_we, req_avail, resp_done: std_logic;
  
  signal moti, moti_udp: trans_moti;
  signal tomi, tomi_udp: trans_tomi;
  signal arb_moti_bus: trans_moti_array(NOOB downto 0);
  signal arb_tomi_bus: trans_tomi_array(NOOB downto 0);


  signal mac_txd_i: std_logic_vector(7 downto 0);
  signal mac_txdvld_i: std_logic;
	signal ipb_out_i : ipb_wbus;

--component icon3
--  PORT (
--    CONTROL0 : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--    CONTROL1 : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--    CONTROL2 : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0));

--end component;
--component ila128x4096
--  PORT (
--    CONTROL : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--    CLK : IN STD_LOGIC;
--    DATA : IN STD_LOGIC_VECTOR(143 DOWNTO 0);
--    TRIG0 : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
--    TRIG1 : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
--    TRIG2 : IN STD_LOGIC_VECTOR(7 DOWNTO 0));

--end component;
--component vio64
--  PORT (
--    CONTROL : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--    ASYNC_IN : IN STD_LOGIC_VECTOR(63 DOWNTO 0));

--end component;
--signal cs0_control : std_logic_vector(35 downto 0) := (others => '0');
--signal cs1_control : std_logic_vector(35 downto 0) := (others => '0');
--signal cs2_control : std_logic_vector(35 downto 0) := (others => '0');
--signal cs0_data : std_logic_vector(143 downto 0) := (others => '0');
--signal cs1_data : std_logic_vector(143 downto 0) := (others => '0');
--signal async_in : std_logic_vector(63 downto 0) := (others => '0');
--signal cs0_trig0 : std_logic_vector(7 downto 0) := (others => '0');
--signal cs0_trig1 : std_logic_vector(7 downto 0) := (others => '0');
--signal cs0_trig2 : std_logic_vector(7 downto 0) := (others => '0');
--signal cs1_trig0 : std_logic_vector(7 downto 0) := (others => '0');
--signal cs1_trig1 : std_logic_vector(7 downto 0) := (others => '0');
--signal cs1_trig2 : std_logic_vector(7 downto 0) := (others => '0');

begin
--icon : icon3
--  port map (
--    CONTROL0 => cs0_control,
--    CONTROL1 => cs1_control,
--    CONTROL2 => cs2_control);
--cs0 : ila128x4096
--  port map (
--    CONTROL => cs0_control,
--    CLK => mac_txclk,
--    DATA => cs0_data,
--    TRIG0 => cs0_trig0,
--    TRIG1 => cs0_trig1,
--    TRIG2 => cs0_trig2
--		);
--cs0_data(0) <= udp_ready;
--cs0_data(1) <= udp_done;
--cs0_data(2) <= udp_xmit;
--cs0_data(3) <= udp_we;
--cs0_data(4) <= udp_space;
--cs0_data(5) <= udp_xmit_req;
--cs0_data(6) <= udp_xmit_ok;
--cs0_data(17 downto 7) <= udp_rxa;
--cs0_data(28 downto 18) <= udp_txa;
--cs0_data(39 downto 29) <= udp_len;
--cs0_data(47 downto 40) <= udp_txd;
--cs0_data(58 downto 48) <= packet_rxa;
--cs0_data(69 downto 59) <= packet_txa;
--cs0_data(80 downto 70) <= packet_txlen;
--cs0_data(88 downto 81) <= packet_txd;
--cs0_data(96 downto 89) <= packet_rxd;
--cs0_data(97) <= packet_rxready;
--cs0_data(98) <= packet_rxdone;
--cs0_data(99) <= packet_txwe;
--cs0_data(100) <= packet_txdone;
--cs0_data(101) <= mac_rxdvld;
--cs0_data(102) <= mac_rxgoodframe;
--cs0_data(103) <= mac_rxbadframe;
--cs0_data(111 downto 104) <= mac_rxd;
--cs0_data(112) <= mac_txdvld_i;
--cs0_data(113) <= mac_txack;
--cs0_data(121 downto 114) <= mac_txd_i;
--cs0_data(132 downto 122) <= packet_rxl;
--cs0_data(143 downto 133) <= (others => '0');
--cs0_trig0(0) <= mac_rxdvld;
--cs0_trig0(1) <= mac_txdvld_i;
--cs0_trig0(2) <= packet_rxready;
--cs0_trig0(3) <= packet_rxdone;
--cs0_trig0(4) <= udp_ready;
--cs0_trig0(5) <= udp_we;
--cs0_trig0(7 downto 6) <= (others => '0');
--cs0_trig1(0) <= mac_rxdvld;
--cs0_trig1(1) <= mac_txdvld_i;
--cs0_trig1(2) <= packet_rxready;
--cs0_trig1(3) <= packet_rxdone;
--cs0_trig0(4) <= udp_ready;
--cs0_trig0(5) <= udp_we;
--cs0_trig0(7 downto 6) <= (others => '0');
--cs0_trig2(0) <= mac_rxdvld;
--cs0_trig2(1) <= mac_txdvld_i;
--cs0_trig2(2) <= packet_rxready;
--cs0_trig2(3) <= packet_rxdone;
--cs0_trig2(4) <= udp_ready;
--cs0_trig2(5) <= udp_we;
--cs0_trig2(7 downto 6) <= (others => '0');
--cs1 : ila128x4096
--  port map (
--    CONTROL => cs1_control,
--    CLK => mac_txclk,
--    DATA => cs1_data,
--    TRIG0 => cs1_trig0,
--    TRIG1 => cs1_trig1,
--    TRIG2 => cs1_trig2
--		);
--cs1_data(0) <= ipb_out_i.ipb_strobe;
--cs1_data(1) <= moti_udp.addr_rst;
--cs1_data(2) <= moti_udp.ready;
--cs1_data(34 downto 3) <= moti_udp.rdata;
--cs1_data(35) <= tomi_udp.we;
--cs1_data(36) <= tomi_udp.done;
--cs1_data(46 downto 37) <= tomi_udp.waddr;
--cs1_data(56 downto 47) <= tomi_udp.raddr;
--cs1_data(88 downto 57) <= tomi_udp.wdata;
--cs1_data(89) <= ipb_in.ipb_ack;
--cs1_data(90) <= ipb_out_i.ipb_write;
--cs1_data(91) <= ipb_in.ipb_err;
--cs1_data(123 downto 92) <= ipb_out_i.ipb_addr;
--cs1_data(132 downto 124) <= packet_req_addr;
--cs1_data(133) <= req_avail;
--cs1_data(143 downto 134) <= ipb_out_i.ipb_wdata(9 downto 0);
--cs1_trig0(0) <= moti_udp.ready;
--cs1_trig0(1) <= tomi_udp.we;
--cs1_trig0(2) <= mac_rxdvld;
--cs1_trig0(3) <= ipb_out_i.ipb_strobe;
--cs1_trig0(7 downto 4) <= (others => '0');
--cs1_trig1(0) <= moti_udp.ready;
--cs1_trig1(1) <= tomi_udp.we;
--cs1_trig1(2) <= tomi_udp.done;
--cs1_trig1(3) <= ipb_out_i.ipb_strobe;
--cs1_trig1(7 downto 4) <= (others => '0');
--cs1_trig2(0) <= moti_udp.ready;
--cs1_trig2(1) <= tomi_udp.we;
--cs1_trig2(2) <= tomi_udp.done;
--cs1_trig2(3) <= ipb_out_i.ipb_strobe;
--cs1_trig2(7 downto 4) <= (others => '0');
--vio : vio64
--  port map (
--    CONTROL => cs2_CONTROL,
--    ASYNC_IN => ASYNC_IN);
--cs0_data(0) <= rst_macclk;
--cs0_data(1) <= icmp_we;
--cs0_data(2) <= icmp_xmit;
--cs0_data(3) <= mac_txdvld_i;
--cs0_data(11 downto 4) <= mac_txd_i;
--cs0_data(21 downto 12) <= icmp_rxa;
--cs0_data(31 downto 22) <= icmp_txa;
--cs0_data(41 downto 32) <= icmp_len;
--cs0_data(42) <= udp_we;
--cs0_data(43) <= udp_xmit;
--cs0_data(44) <= udp_done;
--cs0_data(45) <= udp_xmit_req;
--cs0_data(46) <= udp_space;
--cs0_data(47) <= udp_ready;
--cs0_data(48) <= udp_xmit_ok;
--cs0_data(63 downto 56) <= udp_txd;
--cs0_data(55 downto 49) <= (others => '0');
--cs1_data(0) <= mac_rxdvld;
--cs1_data(1) <= mac_rxgoodframe;
--cs1_data(2) <= packet_rxready;
--cs1_data(3) <= packet_rxdone;
----cs1_data(0) <= ;
--cs1_data(11 downto 4) <= mac_rxd;
--cs1_data(19 downto 12) <= packet_rxd;
--cs1_data(30 downto 20) <= packet_rxa;
--cs1_data(41 downto 31) <= packet_rxl;
--cs1_data(49 downto 42) <= packet_txd;
--cs1_data(50) <= packet_txwe;
--cs1_data(51) <= packet_txdone;
--cs1_data(52) <= moti_udp.ready;
--cs1_data(53) <= tomi_udp.we;
--cs1_data(54) <= tomi_udp.done;
--cs1_data(63 downto 55) <= (others => '0');
--cs0_trig(0) <= mac_rxdvld;
--cs0_trig(7 downto 1) <= (others => '0');
--cs1_trig(0) <= mac_rxdvld;
--cs1_trig(7 downto 1) <= (others => '0');
--async_in(31 downto 0) <= ip_addr;
mac_txdvld <= mac_txdvld_i;
ipb_out <= ipb_out_i;
mac_txd <= mac_txd_i;
  txbuf: gbe_txpacketbuffer port map(
    mac_clk => mac_txclk,
    mac_txd => mac_txd_i,
    mac_txdv => mac_txdvld_i,
    mac_txack => mac_txack,
    reset => rst_macclk,
    packet_txd => packet_txd,
    packet_addr => packet_txa,
    packet_we => packet_txwe,
    packet_done => packet_txdone,
    packet_len => packet_txlen
  );

  rxbuf: gbe_rxpacketbuffer port map(
    clk => mac_txclk,
    mac_clk => mac_rxclk,
    mac_rxd => mac_rxd,
    mac_rxdv => mac_rxdvld,
    mac_rxpacketok => mac_rxgoodframe,
    mac_rxpacketbad => mac_rxbadframe,
    reset => rst_macclk,
    packet_rxd => packet_rxd,
    packet_rxa => packet_rxa,
    packet_rxready => packet_rxready,
    packet_rxdone => packet_rxdone,
    packet_len => packet_rxl
  );
  
  handler: packet_handler port map(
    clk => mac_txclk,
    reset => rst_macclk,
    rxa => packet_rxa,
    rxd => packet_rxd,
    txa => packet_txa,
    txd => packet_txd,
    rx_ready => packet_rxready,
    rx_done => packet_rxdone,
    tx_we => packet_txwe,
    tx_done => packet_txdone,
    tx_len => packet_txlen,
    ip => ip_addr,
    arp_rxa => arp_rxa,
    arp_txa => arp_txa,
    arp_len => "101010",
    arp_txd => arp_txd,
    arp_we => arp_we,
    arp_xmit => arp_xmit,
    arp_done => arp_done,
    arp_ready => arp_ready,
    icmp_rxa => icmp_rxa,
    icmp_txa => icmp_txa,
    icmp_len => icmp_len,
    icmp_txd => icmp_txd,
    icmp_we => icmp_we,
    icmp_xmit => icmp_xmit,
    icmp_done => icmp_done,
    icmp_ready => icmp_ready,
    udp_rxa => udp_rxa,
    udp_txa => udp_txa,
    udp_len => udp_len,
    udp_txd => udp_txd,
    udp_we => udp_we,
    udp_xmit => udp_xmit,
    udp_done => udp_done,
    udp_ready => udp_ready,
    udp_space => udp_space,
    udp_xmit_req => udp_xmit_req,
    udp_xmit_ok => udp_xmit_ok
  );

  arp_block: arp port map(
    mac_clk => mac_txclk,
    reset => rst_macclk,
    packet_ready => arp_ready,
    myMAC => mac_addr,
    myIP => ip_addr,
    packet_data => packet_rxd,
    packet_read_addr => arp_rxa,
    done_with_packet => arp_done,
    packet_out => arp_txd,
    packet_out_addr => arp_txa,
    packet_out_we => arp_we,
    packet_xmit => arp_xmit
  );
  
  icmp_block: icmp port map(
    mac_clk => mac_txclk,
    reset => rst_macclk,
    packet_ready => icmp_ready,
    packet_data => packet_rxd,
    packet_read_addr => icmp_rxa,
    done_with_packet => icmp_done,
    packet_out => icmp_txd,
    packet_out_addr => icmp_txa,
    packet_out_we => icmp_we,
    packet_xmit => icmp_xmit,
    packet_out_len => icmp_len,
    reset_reg_out => open
  );

  packet_buffer: sub_packetbuffer port map(
    ipb_clk => ipb_clk,
    mac_clk => mac_txclk,
    reset => rst_macclk,
    incoming_ready => udp_ready,
    done_with_incoming => udp_done,
    incoming_space => udp_space,
    rxa => udp_rxa,
    rxd => packet_rxd,
    rxl => packet_rxl,
    txa => udp_txa,
    txd => udp_txd,
    tx_we => udp_we,
    txl => udp_len,
    tx_send => udp_xmit,
    tx_req => udp_xmit_req,
    tx_ok => udp_xmit_ok,
    req_addr => packet_req_addr,
    req_data => packet_req_data,
    req_len => packet_req_len,
    resp_addr => packet_resp_addr,
    resp_data => packet_resp_data,
    resp_len => packet_resp_len,
    resp_we => resp_we,
    req_avail => req_avail,
    resp_done => resp_done
  );

  shim: entity work.udp_shim port map(
    clk => ipb_clk,
    reset => rst_ipb,
    packet_data_i => packet_req_data,
    packet_len_i => packet_req_len,
    packet_addr_i => packet_req_addr,
    packet_data_o => packet_resp_data,
    packet_addr_o => packet_resp_addr,
    packet_len_o => packet_resp_len,
    packet_we_o => resp_we,
    new_packet => req_avail,
    done => resp_done,
    moti => moti_udp,
    tomi => tomi_udp
  );
  

  trans: entity work.transactor port map(
    clk => ipb_clk, 
    reset => rst_ipb,
    ipb_out => ipb_out_i,
    ipb_in => ipb_in,
    moti => moti_udp,
    tomi => tomi_udp
 );

end rtl;
