-- The state machine which controls the ipbus itself
--
-- Dave Newbold, April 2011
--
-- $Id$

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all; 

library work;
use work.ipbus.all;
library UNISIM;
use UNISIM.VComponents.all;

entity transactor_sm is
  port(
    clk: in std_logic; 
    reset: in std_logic;
    rx_data: in std_logic_vector(31 downto 0); -- input packet data
    rx_ready: in std_logic; -- asserted when valid input packet data is available
    rx_next: out std_logic; -- new input packet data on next cycle when this is asserted
	  tx_func: out std_logic_vector(3 downto 0); -- encodes required transmit function (see transactor_tx)
    byte_order: out std_logic; -- controls byte ordering of input and output packet data
    ipb_out: out ipb_wbus;
    ipb_in: in ipb_rbus
 );
 
end transactor_sm;

architecture rtl of transactor_sm is

  constant TIMEOUT: unsigned(7 downto 0) := X"ff";
  
  constant TRANS_RD: std_logic_vector(4 downto 0) := "00011";
  constant TRANS_WR: std_logic_vector(4 downto 0) := "00100";
  constant TRANS_RMWB: std_logic_vector(4 downto 0) := "00101";
  constant TRANS_RMWS: std_logic_vector(4 downto 0) := "00110";
  constant TRANS_RDN: std_logic_vector(4 downto 0) := "01000";
  constant TRANS_WRN: std_logic_vector(4 downto 0) := "01001";
  constant TRANS_INFO: std_logic_vector(4 downto 0) := "11101";
 
  type state_type is (ST_IDLE, ST_CHECK_HDR, ST_HDR, ST_WR_INFO, ST_ADDR, ST_BUS_CYCLE, ST_RMW_1, ST_RMW_2, ST_LAST, ST_ERROR);
  signal state: state_type;
  signal last_trans_id: unsigned(10 downto 0);
  signal words: unsigned(8 downto 0);
  signal strobe_cnts: unsigned(8 downto 0);
  signal trans_type: std_logic_vector(4 downto 0);
  signal next_word, ack, strobe, write, ready_d, rmw_cyc, rmw_write, first: std_logic;
  signal addr: unsigned(31 downto 0);
  signal timer: unsigned(7 downto 0);
  signal rmw_coeff, rmw_input, rmw_result: std_logic_vector(31 downto 0);
  signal err_code: std_logic_vector(2 downto 0);
--component icon2
--  PORT (
--    CONTROL0 : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--    CONTROL1 : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0));

--end component;
--component ila64
--  PORT (
--    CONTROL : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--    CLK : IN STD_LOGIC;
--    DATA : IN STD_LOGIC_VECTOR(63 DOWNTO 0);
--    TRIG0 : IN STD_LOGIC_VECTOR(7 DOWNTO 0));

--end component;
--component vio64
--  PORT (
--    CONTROL : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--    ASYNC_IN : IN STD_LOGIC_VECTOR(63 DOWNTO 0));

--end component;
--signal CONTROL0 : std_logic_vector(35 downto 0) := (others => '0');
--signal CONTROL1 : std_logic_vector(35 downto 0) := (others => '0');
--signal ipb_data : std_logic_vector(63 downto 0) := (others => '0');
--signal chip_data : std_logic_vector(63 downto 0) := (others => '0');
--signal ASYNC_IN : std_logic_vector(63 downto 0) := (others => '0');
--signal chip_trig : std_logic_vector(7 downto 0) := (others => '0');
  
begin
--i_icon : icon2
--  port map (
--    CONTROL0 => CONTROL0,
--    CONTROL1 => CONTROL1
--		);
--i_ila : ila64
--  port map (
--    CONTROL => CONTROL0, 
--    CLK => CLK,
--    DATA => chip_DATA,
--    TRIG0 => chip_TRIG);
--i_ilab : ila64
--  port map (
--    CONTROL => CONTROL1, 
--    CLK => CLK,
--    DATA => ipb_DATA,
--    TRIG0 => chip_TRIG);
--your_instance_name : vio64
--  port map (
--    CONTROL => CONTROL1,
--    ASYNC_IN => ipb_DATA);

--chip_trig(0) <= '1' when state = ST_ADDR and rx_data(12) = '1' else '0';
--chip_trig(1) <= '1' when state = ST_BUS_CYCLE and write = '0' and words(7 downto 6) /= 0 else '0';
--chip_trig(2) <= '1' when state = ST_BUS_CYCLE and write = '1' and addr(12) = '1' else '0';
--chip_trig(7 downto 3) <= (others => '0');
--process(addr)
--begin
--	for i in 0 to 12 loop
--		ipb_data(i+46) <= addr(i);
--	end loop;
--	for i in 0 to 1 loop
--		ipb_data(i+59) <= words(i);
--	end loop;
--	for i in 0 to 1 loop
--		ipb_data(i+62) <= strobe_cnts(i);
--	end loop;
--end process;
--ipb_data(31 downto 0) <= rx_data;
--ipb_data(32) <= ack;
--ipb_data(33) <= strobe;
--ipb_data(34) <= '1' when state = ST_IDLE else '0';
--ipb_data(35) <= '1' when state = ST_CHECK_HDR else '0';
--ipb_data(36) <= '1' when state = ST_HDR else '0';
--ipb_data(37) <= '1' when state = ST_WR_INFO else '0';
--ipb_data(38) <= '1' when state = ST_ADDR else '0';
--ipb_data(39) <= '1' when state = ST_BUS_CYCLE else '0';
--ipb_data(40) <= '1' when state = ST_RMW_1 else '0';
--ipb_data(41) <= '1' when state = ST_RMW_2 else '0';
--ipb_data(42) <= '1' when state = ST_LAST else '0';
--ipb_data(43) <= '1' when state = ST_ERROR else '0';
--ipb_data(44) <= write;
--ipb_data(45) <= rmw_cyc;
--ipb_data(61) <= next_word;
--g_delay1: for i in 0 to 63 generate
--   SRL16E_inst : SRL16E
--   port map (
--      Q => chip_DATA(i),       -- SRL data output
--      A0 => '1',     -- Select[0] input
--      A1 => '1',     -- Select[1] input
--      A2 => '1',     -- Select[2] input
--      A3 => '1',     -- Select[3] input
--      CE => '1',     -- Clock enable input
--      CLK => clk,   -- Clock input
--      D => ipb_data(i)        -- SRL data input
--   );
--end generate;
  process(clk)
  begin
    if rising_edge(clk) then

      if reset='1' then
        state <= ST_IDLE;
        last_trans_id <= (others => '0');
      else

        case state is

-- Starting state

          when ST_IDLE =>

            byte_order <= '0';
            strobe <= '0';
            next_word <= '0';
            first <= '1';
            
            if rx_ready='1' and ready_d='0' then
              state <= ST_CHECK_HDR;
            else
              state <= ST_IDLE;
            end if;

-- Check for endianness, wait for next word from buffer

          when ST_CHECK_HDR =>

            next_word <= '1';
            if rx_data(31 downto 28)=X"f" then
              byte_order <= '1';
            end if;
             
            state <= ST_HDR;
            
-- Decode header word
            
          when ST_HDR =>
            
            if rx_ready = '0' and first = '0' then
              state <= ST_LAST;
            else
              strobe <= '0';
              trans_type <= rx_data(7 downto 3);

              if rx_data(7 downto 3) = TRANS_RMWB or rx_data(7 downto 3) = TRANS_RMWS then
                words <= to_unsigned(1, 9); -- to correct for issues in software
              else
                words <= unsigned(rx_data(16 downto 8));
              end if;

              rmw_write <= '0';            

              if rx_data(31 downto 28) /= X"1" or rx_data(2) = '1' then
						    err_code <= "000"; -- Bad header
						    state <= ST_ERROR;
--					    elsif rx_data(16 downto 8)/=(8 downto 0 => '0') and unsigned(rx_data(27 downto 17))/=last_trans_id+1 then
--						    err_code <= "001"; -- Non-sequential header ID
--						    state <= ST_ERROR;
					    else
						    if rx_data(16 downto 8) /= (8 downto 0 => '0')then
							    last_trans_id <= unsigned(rx_data(27 downto 17));
						    end if;
						    if rx_data(7 downto 3) = TRANS_INFO then
							    next_word <= '0';
							    state <= ST_WR_INFO;
						    elsif rx_data(16 downto 8) = (8 downto 0 =>'0') then
							    state <= ST_HDR;
						    else
							    state <= ST_ADDR;
						    end if;
              end if;
            end if;
            
            first <= '0';

-- Write last transaction info

          when ST_WR_INFO =>

            next_word <= '1';			
				    state <= ST_HDR;

-- Load address counter
            
          when ST_ADDR =>

            addr <= unsigned(rx_data);
            if trans_type = TRANS_WR or trans_type = TRANS_WRN then
              write <= '1';
							next_word <= '1';
            else
              write <= '0';
							next_word <= '0';
            end if;
            
            words <= words - 1;
            timer <= (others => '0');
						strobe <= '1';
						strobe_cnts <= words - 1;
                        
            state <= ST_BUS_CYCLE;

-- The bus transaction

          when ST_BUS_CYCLE =>
						if(strobe_cnts = 0)then
							strobe <= '0';
							next_word <= '0';
						else
							strobe_cnts <= strobe_cnts - 1;
						  if trans_type = TRANS_RD or trans_type = TRANS_WR then
								addr <= addr + 1;
							end if;
						end if;
            if ipb_in.ipb_err = '1' then
              strobe <= '0';
		          err_code <= "01" & write; -- Bus error
              state <= ST_ERROR;
            elsif ack = '1' then

              rmw_input <= ipb_in.ipb_rdata;

              if words /= 0 then
                words <= words - 1;
                timer <= (others => '0');
                state <= ST_BUS_CYCLE;
              else
                next_word <= '1';
                if rmw_cyc = '1' and rmw_write = '0' then
                  state <= ST_RMW_1;
                else
                  state <= ST_HDR;
                end if;
              end if;
              
            else
              timer <= timer + 1;
              if timer = TIMEOUT then
					      err_code <= "10" & write; -- Bus timeout
                state <= ST_ERROR;
              else
                state <= ST_BUS_CYCLE;
              end if;
            end if;
            
          when ST_RMW_1 =>
            
            timer <= (others => '0');
            rmw_coeff <= rx_data;
            write <= '1';
            rmw_write <= '1';
            
            if trans_type = TRANS_RMWB then
              state <= ST_RMW_2;
            else
              rmw_result <= std_logic_vector(unsigned(rmw_input) + unsigned(rx_data));
              next_word <= '0';
              strobe <= '1';
              state <= ST_BUS_CYCLE;
            end if;
            
          when ST_RMW_2 =>
            
            rmw_result <= (rmw_input and rmw_coeff) or rx_data;
            next_word <= '0';
            strobe <= '1';
            state <= ST_BUS_CYCLE;

          when ST_LAST =>
                    
            state <= ST_IDLE;

-- Error has occured
                                          
          when ST_ERROR =>

            state <= ST_LAST;

          end case;
      end if;
    end if; 
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      ready_d <= rx_ready;
    end if;
  end process;
  
  rmw_cyc <= '1' when trans_type = TRANS_RMWB or trans_type = TRANS_RMWS else '0';
  
  ack <= ipb_in.ipb_ack or ipb_in.ipb_err;
--  rx_next <= next_word or (ack and write and not rmw_cyc);
  rx_next <= next_word;
  
  with state select tx_func <=
	'1' & err_code when ST_ERROR,
	"0111" when ST_LAST,
	"0100" when ST_HDR,
	"0101" when ST_WR_INFO,
  "00" & (ack and (rmw_cyc xor not write)) & (ack and not (rmw_cyc and not write)) when ST_BUS_CYCLE,
	"0000" when others;

  ipb_out.ipb_addr <= std_logic_vector(addr);
  ipb_out.ipb_write <= write;
  ipb_out.ipb_strobe <= strobe;
 
	ipb_out.ipb_wdata <= rx_data when rmw_cyc='0' else rmw_result;
  
end rtl;
