----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    08:04:48 08/17/2012 
-- Design Name: 
-- Module Name:    RAM48KB - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
-- Readout latency from addrb change to doutb is two clkb cycles if enb is '1'
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity SDP8KX65 is
    Port ( wclk : in  STD_LOGIC;
           we : in  STD_LOGIC;
           waddr : in  STD_LOGIC_VECTOR (12 downto 0);
           din : in  STD_LOGIC_VECTOR (64 downto 0);
           rclk : in  STD_LOGIC;
           rden : in  STD_LOGIC;
           regce : in  STD_LOGIC;
           raddr : in  STD_LOGIC_VECTOR (12 downto 0);
           dout : out  STD_LOGIC_VECTOR (64 downto 0) := (others => '0'));
end SDP8KX65;

architecture Behavioral of SDP8KX65 is
signal doutl : std_logic_vector(8 downto 0) := (others => '0');
signal douth : std_logic_vector(8 downto 0) := (others => '0');
signal wel : std_logic_vector(0 downto 0) := (others => '0');
signal weh : std_logic_vector(0 downto 0) := (others => '0');
signal raddr12_q : std_logic := '0';
begin
g_buf: for i in 0 to 13 generate
   i_buf : BRAM_SDP_MACRO
   generic map (
      BRAM_SIZE => "36Kb", -- Target BRAM, "18Kb" or "36Kb" 
      DEVICE => "7SERIES", -- Target device: "VIRTEX5", "VIRTEX6", "7SERIES", "SPARTAN6" 
      WRITE_WIDTH => 4,    -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      READ_WIDTH => 4,     -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      DO_REG => 1, -- Optional output register (0 or 1)
      INIT_FILE => "NONE",
      SIM_COLLISION_CHECK => "NONE") -- Collision check enable "ALL", "WARNING_ONLY", 
                                    -- "GENERATE_X_ONLY" or "NONE"       
   port map (
      DO => dout(4*i+3 downto 4*i),         -- Output read data port, width defined by READ_WIDTH parameter
      DI => din(4*i+3 downto 4*i),         -- Input write data port, width defined by WRITE_WIDTH parameter
      RDADDR => raddr, -- Input read address, width defined by read port depth
      RDCLK => rclk,   -- 1-bit input read clock
      RDEN => rden,     -- 1-bit input read port enable
      REGCE => regce,   -- 1-bit input read output register enable
      RST => '0',       -- 1-bit input reset 
      WE => "1",         -- Input write enable, width defined by write port depth
      WRADDR => waddr, -- Input write address, width defined by write port depth
      WRCLK => wclk,   -- 1-bit input write clock
      WREN => we      -- 1-bit input write port enable
   );
end generate;
i_bufl : BRAM_SDP_MACRO
   generic map (
      BRAM_SIZE => "36Kb", -- Target BRAM, "18Kb" or "36Kb" 
      DEVICE => "7SERIES", -- Target device: "VIRTEX5", "VIRTEX6", "7SERIES", "SPARTAN6" 
      WRITE_WIDTH => 9,    -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      READ_WIDTH => 9,     -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      DO_REG => 0, -- Optional output register (0 or 1)
      INIT_FILE => "NONE",
      SIM_COLLISION_CHECK => "NONE") -- Collision check enable "ALL", "WARNING_ONLY", 
                                    -- "GENERATE_X_ONLY" or "NONE"       
   port map (
      DO => doutl,         -- Output read data port, width defined by READ_WIDTH parameter
      DI => din(64 downto 56),         -- Input write data port, width defined by WRITE_WIDTH parameter
      RDADDR => raddr(11 downto 0), -- Input read address, width defined by read port depth
      RDCLK => rclk,   -- 1-bit input read clock
      RDEN => rden,     -- 1-bit input read port enable
      REGCE => '1',   -- 1-bit input read output register enable
      RST => '0',       -- 1-bit input reset 
      WE => wel,         -- Input write enable, width defined by write port depth
      WRADDR => waddr(11 downto 0), -- Input write address, width defined by write port depth
      WRCLK => wclk,   -- 1-bit input write clock
      WREN => we      -- 1-bit input write port enable
   );
wel(0) <= not waddr(12);
i_bufh : BRAM_SDP_MACRO
   generic map (
      BRAM_SIZE => "36Kb", -- Target BRAM, "18Kb" or "36Kb" 
      DEVICE => "7SERIES", -- Target device: "VIRTEX5", "VIRTEX6", "7SERIES", "SPARTAN6" 
      WRITE_WIDTH => 9,    -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      READ_WIDTH => 9,     -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      DO_REG => 0, -- Optional output register (0 or 1)
      INIT_FILE => "NONE",
      SIM_COLLISION_CHECK => "NONE") -- Collision check enable "ALL", "WARNING_ONLY", 
                                    -- "GENERATE_X_ONLY" or "NONE"       
   port map (
      DO => douth,         -- Output read data port, width defined by READ_WIDTH parameter
      DI => din(64 downto 56),         -- Input write data port, width defined by WRITE_WIDTH parameter
      RDADDR => raddr(11 downto 0), -- Input read address, width defined by read port depth
      RDCLK => rclk,   -- 1-bit input read clock
      RDEN => rden,     -- 1-bit input read port enable
      REGCE => '1',   -- 1-bit input read output register enable
      RST => '0',       -- 1-bit input reset 
      WE => weh,         -- Input write enable, width defined by write port depth
      WRADDR => waddr(11 downto 0), -- Input write address, width defined by write port depth
      WRCLK => wclk,   -- 1-bit input write clock
      WREN => we      -- 1-bit input write port enable
   );
weh(0) <= waddr(12);
process(rclk)
begin
	if(rclk'event and rclk = '1')then
		if(rden = '1')then
			raddr12_q <= raddr(12);
		end if;
		if(regce = '1')then
			if(raddr12_q = '0')then
				dout(64 downto 56) <= doutl;
			else
				dout(64 downto 56) <= douth;
			end if;
		end if;
	end if;
end process;
end Behavioral;

